@@ -6,16 +6,8 @@ COPYRIGHT distributed with the source files.
 
 */
 
-#include "config.h"
-
-// deal with any configuration issues arising from config.h
-
-#ifndef HAVE_SSIZE_T
-typedef signed int ssize_t;
-#endif
-
-
 #include <iostream>
+#include <ostream>
 #include <string>
 #include <cstdlib>
 #include <cstring>
