@@ -91,9 +91,7 @@ const char *Usage =
 #ifdef ENABLE_NLS
 #include <locale.h>
 #include <libintl.h>
-#include <glib/gconvert.h>
-#include <glib/gunicode.h>
-#include <glib/gmem.h>
+#include <glib.h>
 #endif
 
 #include "efaxio.h"		/* EFAX */
