@@ -14,9 +14,7 @@
 #include <config.h>             /* For NLS */
 #ifdef ENABLE_NLS
 #include <libintl.h>
-#include <glib/gconvert.h>
-#include <glib/gunicode.h>
-#include <glib/gmem.h>
+#include <glib.h>
 #endif
 
 #ifndef SEEK_SET
