@@ -14,6 +14,13 @@
 
 #include "efaxmsg.h"
 
+// there can be certain circumstances where PIPE_BUF is not
+// defined in <limits.h>.  If so, just define the minimum
+// required by POSIX
+#ifndef PIPE_BUF
+#define PIPE_BUF 512
+#endif
+
 #define MAXTSTAMP 80		/* maximum length of a time stamp */
 
 #if PIPE_BUF>8192
@@ -140,14 +147,13 @@ int msg ( char *fmt, ... )
   int err=0, i, flags=0 ;
   char *p;
 
+  va_list ap ;
+
 #ifdef ENABLE_NLS
   gsize written = 0 ;
   char *message ;
 #endif
 
-  va_list ap ;
-  va_start ( ap, fmt ) ;
-
   if ( ! init ) {
     logfile[0] = stderr ;
     logfile[1] = stdout ;
@@ -180,6 +186,7 @@ int msg ( char *fmt, ... )
   }
   
   for ( i=0 ; i<NLOG ; i++ ) {
+    va_start ( ap, fmt ) ;
 
     for ( p=fmt ; *p ; p++ ) {
       switch ( *p ) {
@@ -236,10 +243,9 @@ int msg ( char *fmt, ... )
       
     }
     
+    va_end ( ap ) ;
   }
   
-  va_end ( ap ) ;
-  
   return err ;
 }
 
