@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -22,15 +22,12 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/emitter.h>
 
 
-namespace { // we put the functions in anonymous namespace in addressbook.cpp
-            // so they are not exported at link time
 namespace AddressBookCB {
   extern "C" {
     void addr_book_button_clicked(GtkWidget*, void*);
     void addr_book_drag_n_drop(GtkTreeModel*, GtkTreePath*, void*);
   }
 }
-}
 
 class AddressBook: public WinBase {
   static int is_address_list;
@@ -71,15 +68,12 @@ public:
   ~AddressBook(void);
 };
 
-namespace { // we put the functions in anonymous namespace in addressbook.cpp
-            // so they are not exported at link time
 namespace AddressDialogCB {
   extern "C" {
     void addr_dialog_selected(GtkWidget*, void*);
     gboolean addr_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
   }
 }
-}
 
 class AddressDialog: public WinBase {
 
