@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -127,7 +127,8 @@ std::pair<const char*, char* const*> FileReadSelectDialog::get_view_file_parms(v
   }
   catch (Utf8::ConversionError&) {
     write_error("UTF-8 conversion error in FileReadSelectDialog::get_view_file_parms()\n");
-    return std::pair<const char*, char* const*>(0,0);
+    return std::pair<const char*, char* const*>(static_cast<char*>(0),
+						static_cast<char* const*>(0));
   }
 
   if ((end_pos = view_cmd.find_first_of(' ')) != std::string::npos) { // we have parms
@@ -270,7 +271,7 @@ gboolean GplDialogCB::gpl_dialog_key_press_event(GtkWidget*, GdkEventKey* event_
   GplDialog* instance_p = static_cast<GplDialog*>(data);
   int keycode = event_p->keyval;
   
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   if (keycode == GDK_KEY_Escape) instance_p->on_delete_event();
   else if (keycode == GDK_KEY_Home || keycode == GDK_KEY_End
 	   || keycode == GDK_KEY_Up || keycode == GDK_KEY_Down
@@ -288,37 +289,6 @@ gboolean GplDialogCB::gpl_dialog_key_press_event(GtkWidget*, GdkEventKey* event_
   return true;    // stop processing here
 }
 
-void GplDialogCB::gpl_dialog_style_set(GtkWidget*, GtkStyle*, void* data) {
-
-  GplDialog* instance_p = static_cast<GplDialog*>(data);
-
-  //now set the buttons in GplDialog
-  int width = 0;
-  int height = 0;
-  // create a button to work on - the GobjHandle object will claim ownership of the button
-  GobjHandle<GtkWidget> button_h(gtk_button_new());
-
-  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(instance_p->get_win()));
-  gtk_widget_set_style(button_h, style_p);
-
-  GobjHandle<PangoLayout> pango_layout_h(gtk_widget_create_pango_layout(button_h,
-							 instance_p->max_text.c_str()));
-  pango_layout_get_pixel_size(pango_layout_h, &width, &height);
-
-  // add padding
-  width += 18;
-  height += 12;
-
-  // have some sensible minimum width and height if a very small font chosen
-  const int min_width = instance_p->standard_size * 4;
-  const int min_height = 30;
-  if (width < min_width) width = min_width;
-  if (height < min_height) height = min_height;
-
-  gtk_widget_set_size_request(instance_p->accept_button_p, width, height);
-  gtk_widget_set_size_request(instance_p->reject_button_p, width, height);
-}
-
 GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices and Disclaimers"),
 				        prog_config.window_icon_h,
 					true),
@@ -338,7 +308,11 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
 
   PangoFontDescription* font_description = 
     pango_font_description_from_string(prog_config.fixed_font.c_str());
+#if GTK_CHECK_VERSION(2,99,0)
+  gtk_widget_override_font(GTK_WIDGET(text_view_p), font_description);
+#else
   gtk_widget_modify_font(GTK_WIDGET(text_view_p), font_description);
+#endif
   pango_font_description_free(font_description);
 
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(text_view_p),
@@ -351,8 +325,6 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
 				 GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(scrolled_window_p), GTK_WIDGET(text_view_p));
 
-  get_longest_button_text();
-
   gtk_table_attach(table_p, GTK_WIDGET(scrolled_window_p),
 		   0, 2, 0, 1,
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND),
@@ -373,8 +345,10 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
 		   GTK_SHRINK, GTK_SHRINK,
 		   0, standard_size/3);
 
-  g_signal_connect(G_OBJECT(get_win()), "style_set",
-		   G_CALLBACK(GplDialogCB::gpl_dialog_style_set), this);
+  GobjHandle<GtkSizeGroup> group_h(gtk_size_group_new(GTK_SIZE_GROUP_BOTH));
+  gtk_size_group_add_widget(group_h, accept_button_p);
+  gtk_size_group_add_widget(group_h, reject_button_p);
+
   g_signal_connect(G_OBJECT(accept_button_p), "clicked",
 		   G_CALLBACK(GplDialogCB::gpl_dialog_selected), this);
   g_signal_connect(G_OBJECT(reject_button_p), "clicked",
@@ -414,35 +388,6 @@ void GplDialog::on_delete_event(void) {
   close();
 }
 
-void GplDialog::get_longest_button_text(void) {
-  std::vector<std::string> text_vec;
-  text_vec.push_back(gettext("Accept"));
-  text_vec.push_back(gettext("Reject"));
-
-  // create a button to work on - the GobjHandle object will claim ownership of the button
-  GobjHandle<GtkWidget> button_h(gtk_button_new());
-
-  std::vector<std::string>::const_iterator temp_iter;
-  std::vector<std::string>::const_iterator max_width_iter;
-  int width;
-  int height;
-  int max_width;
-  for (temp_iter = text_vec.begin(), max_width_iter = text_vec.begin(),
-	 width = 0, height = 0, max_width = 0;
-       temp_iter != text_vec.end(); ++temp_iter) {
-
-    GobjHandle<PangoLayout> pango_layout_h(gtk_widget_create_pango_layout(button_h,
-							      temp_iter->c_str()));
-    pango_layout_get_pixel_size(pango_layout_h, &width, &height);
-
-    if (width > max_width) {
-      max_width = width;
-      max_width_iter = temp_iter;
-    }
-  }
-  max_text = *max_width_iter;
-}
-
 
 void InfoDialogCB::info_dialog_selected(GtkDialog*, int, void* data) {
   static_cast<InfoDialog*>(data)->close();
@@ -502,7 +447,7 @@ PromptDialog::PromptDialog(const char* text, const char* caption,
                            WinBase(caption, prog_config.window_icon_h,
 				   modal, parent_p,
 				   GTK_WINDOW(gtk_dialog_new_with_buttons(0, 0,
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
 						  GtkDialogFlags(0),
 #else
 						  GTK_DIALOG_NO_SEPARATOR,
@@ -569,7 +514,7 @@ AboutEfaxGtkDialog::AboutEfaxGtkDialog(GtkWindow* parent_p, bool modal):
   gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(get_win()), "efax-gtk");
   gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(get_win()), VERSION);
   gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(get_win()),
-				 gettext("Copyright (C) 2001 - 2010 Chris Vine"));
+				 gettext("Copyright (C) 2001 - 2011 Chris Vine"));
   gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(get_win()),
 			       gpl_copyright_msg()->c_str()); // this function calls g_strdup()
   gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(get_win()),
