@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -56,7 +56,6 @@ namespace GplDialogCB {
   extern "C" {
     void gpl_dialog_selected(GtkWidget*, void*);
     gboolean gpl_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
-    void gpl_dialog_style_set(GtkWidget*, GtkStyle*, void*);
   }
 }
 
@@ -69,15 +68,12 @@ private:
   GtkWidget* accept_button_p;
   GtkWidget* reject_button_p;
   GtkTextView* text_view_p;
-  std::string max_text;
-  void get_longest_button_text(void);
 protected:
   virtual int get_exec_val(void) const;
   virtual void on_delete_event(void);
 public:
   friend void GplDialogCB::gpl_dialog_selected(GtkWidget*, void*);
   friend gboolean GplDialogCB::gpl_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
-  friend void GplDialogCB::gpl_dialog_style_set(GtkWidget*, GtkStyle*, void*);
 
   GplDialog(int standard_size);
 };
