@@ -41,6 +41,7 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/mem_fun.h>
 #include <c++-gtk-utils/convert.h>
 #include <c++-gtk-utils/timeout.h>
+#include <c++-gtk-utils/cgu_config.h>
 
 #ifdef HAVE_STREAM_IMBUE
 #include <locale>
@@ -210,10 +211,8 @@ void EfaxController::sendfax_impl(const Fax_item& fax_item,
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
-    std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(Callback::make(*this, &EfaxController::sendfax_thread, in_redial_cb),
-			    false);
-    if (!res.get()) {
+    if (!Thread::Thread::start(Callback::make(*this, &EfaxController::sendfax_thread, in_redial_cb),
+			       false).get()) {
       write_error("Cannot start thread to make fax for sending\n");
       stdout_pipe.close();
       state = inactive;
@@ -625,11 +624,9 @@ void EfaxController::receive(State mode) {
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
     const char* thread_arg = fax_pathname_h.release();
-    std::auto_ptr<Thread::Thread> res =
-      Thread::Thread::start(Callback::make(*this, &EfaxController::receive_thread,
-					   thread_arg),
-			    false);
-    if (!res.get()) {
+    if (!Thread::Thread::start(Callback::make(*this, &EfaxController::receive_thread,
+					      thread_arg),
+			       false).get()) {
       write_error("Cannot start thread to receive fax\n");
       stdout_pipe.close();
       state = inactive;
