@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2007, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -29,11 +29,8 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/async_queue.h>
 
 
-namespace { // we put the function in anonymous namespace in fax_list.cpp
-            // so it is not exported at link time
 namespace FaxListDialogCB {
-  extern "C" void fax_list_button_clicked(GtkWidget*, void*);
-}
+  extern "C" void fax_list_dialog_button_clicked(GtkWidget*, void*);
 }
 
 class FaxListDialog: public WinBase {
@@ -81,7 +78,7 @@ class FaxListDialog: public WinBase {
   void view_fax_thread(std::string*, std::pair<std::string*, int>);
   void delete_parms(std::pair<const char*, char* const*>);
 public:
-  friend void FaxListDialogCB::fax_list_button_clicked(GtkWidget*, void*);
+  friend void FaxListDialogCB::fax_list_dialog_button_clicked(GtkWidget*, void*);
 
   Releaser releaser;
   EmitterArg<int&> get_new_fax_count_sig;
@@ -103,15 +100,12 @@ public:
 };
 
 
-namespace { // we put the functions in anonymous namespace in fax_list.cpp
-            // so they are not exported at link time
 namespace EntryDialogCB {
   extern "C" {
     void entry_dialog_selected(GtkWidget*, void*);
     gboolean entry_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
   }
 }
-}
 
 class EntryDialog: public WinBase {
 
