@@ -50,28 +50,6 @@ public:
   std::pair<bool, std::string> validate(const std::string&);
 };
 
-namespace { // we put the functions in anonymous namespace in fax_list_manager.cpp
-            // so they are not exported at link time
-
-namespace FaxListManagerCB {
-  extern "C" {
-    void folder_tree_view_drag_begin(GtkWidget*, GdkDragContext*, void*);
-    void fax_tree_view_drag_begin(GtkWidget*, GdkDragContext*, void*);
-    gboolean drag_motion(GtkWidget*, GdkDragContext*, gint, gint, guint, void*);
-    gboolean folder_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
-    gboolean fax_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
-    gboolean fax_tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
-    void drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData*,
-		       guint, guint, void*);
-    void drag_data_received(GtkWidget*, GdkDragContext*, gint, gint,
-			    GtkSelectionData*, guint, guint, void*);
-    void fax_selected(GtkTreeSelection*, void*);
-    void folder_selected(GtkTreeSelection*, void*);
-    void toggle_sort_type(GtkTreeViewColumn*, void*);
-  } // extern "C"
-}   // namespace FaxListManagerCB
-}   // anonymous namespace
-
 class FaxListManager {
 
   static bool fax_received_list_main_iteration;
@@ -123,19 +101,8 @@ class FaxListManager {
   void write_path_timer_cb(bool&) {write_path();}
 
 public:
-  friend void FaxListManagerCB::folder_tree_view_drag_begin(GtkWidget*, GdkDragContext*, void*);
-  friend void FaxListManagerCB::fax_tree_view_drag_begin(GtkWidget*, GdkDragContext*, void*);
-  friend gboolean FaxListManagerCB::drag_motion(GtkWidget*, GdkDragContext*, gint, gint, guint, void*);
-  friend gboolean FaxListManagerCB::folder_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
-  friend gboolean FaxListManagerCB::fax_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
-  friend gboolean FaxListManagerCB::fax_tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
-  friend void FaxListManagerCB::drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData*,
-					      guint, guint, void*);
-  friend void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext*, gint, gint,
-						   GtkSelectionData*, guint, guint, void*);
-  friend void FaxListManagerCB::fax_selected(GtkTreeSelection*, void*);
-  friend void FaxListManagerCB::folder_selected(GtkTreeSelection*, void*);
-  friend void FaxListManagerCB::toggle_sort_type(GtkTreeViewColumn*, void*);
+  class CB;
+  friend class CB;
 
   Emitter selection_notify;
   Emitter fax_double_click_notify;
