@@ -452,7 +452,8 @@ std::pair<const char*, char* const*> FileListDialog::get_view_file_parms(const s
   }
   catch (Utf8::ConversionError&) {
     write_error("UTF-8 conversion error in FileListDialog::get_view_file_parms()\n");
-    return std::pair<const char*, char* const*>(0,0);
+    return std::pair<const char*, char* const*>(static_cast<char*>(0),
+						static_cast<char* const*>(0));
   }
 
   if ((end_pos = view_cmd.find_first_of(' ')) != std::string::npos) { // we have parms
