@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2005, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -21,15 +21,12 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/emitter.h>
 
 
-namespace { // we put the functions in anonymous namespace in file_list.cpp
-            // so they are not exported at link time
 namespace FileListDialogCB {
   extern "C" {
-    void file_list_button_clicked(GtkWidget*, void*);
-    void file_list_set_buttons(GtkTreeSelection*, void* data);
+    void file_list_dialog_button_clicked(GtkWidget*, void*);
+    void file_list_dialog_set_buttons(GtkTreeSelection*, void* data);
   }
 }
-}
 
 class FileListDialog: public WinBase {
   static int is_file_list;
@@ -57,8 +54,8 @@ class FileListDialog: public WinBase {
   std::pair<const char*, char* const*> get_view_file_parms(const std::string&);
   void delete_parms(std::pair<const char*, char* const*>);
 public:
-  friend void FileListDialogCB::file_list_button_clicked(GtkWidget*, void*);
-  friend void FileListDialogCB::file_list_set_buttons(GtkTreeSelection*, void*);
+  friend void FileListDialogCB::file_list_dialog_button_clicked(GtkWidget*, void*);
+  friend void FileListDialogCB::file_list_dialog_set_buttons(GtkTreeSelection*, void*);
 
   EmitterArg<const std::string&> accepted;
 
