@@ -4,13 +4,15 @@
 #include <string>
 #include <memory>
 
+#include <c++-gtk-utils/cgu_config.h>
+
 const char copyright_msg_part_1[] = 
 
 "\n\n"
 
 "-------------------------------------------------------------------------\n"
 "\n"
-"Copyright (C) 2001 to 2010 Chris Vine\n"
+"Copyright (C) 2001 to 2011 Chris Vine\n"
 "\n"
 "\n"
 "    This program is free software; you can redistribute it and/or modify\n"
@@ -345,9 +347,15 @@ const char copyright_msg_part_7[] =
 "		     END OF TERMS AND CONDITIONS\n"
 ;
 
+// if c++-gtk-utils-2.0 is being used, we know we are
+// compiling under C++0x/C++11
+#if CGU_API_VERSION>=20
+inline std::unique_ptr<std::string> gpl_copyright_msg(void) {
+  std::unique_ptr<std::string> return_val(new std::string(copyright_msg_part_1));
+#else
 inline std::auto_ptr<std::string> gpl_copyright_msg(void) {
-
   std::auto_ptr<std::string> return_val(new std::string(copyright_msg_part_1));
+#endif
   return_val->append(copyright_msg_part_2);
   return_val->append(copyright_msg_part_3);
   return_val->append(copyright_msg_part_4);
