@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2005 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -16,15 +16,12 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/window.h>
 
 
-namespace { // we put the functions in anonymous namespace in helpfile.cpp
-            // so they are not exported at link time
 namespace HelpDialogCB {
   extern "C" {
-    void help_button_clicked(GtkWidget*, void*);
-    gboolean help_key_press_event(GtkWidget*, GdkEventKey*, void*);
+    void help_dialog_button_clicked(GtkWidget*, void*);
+    gboolean help_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
   }
 }
-}   // anonymous namespace
 
 class HelpDialog: public WinBase {
   static int is_helpfile;
@@ -39,8 +36,8 @@ class HelpDialog: public WinBase {
   GtkWidget* make_text_view(const char* text);
   GtkWidget* make_scrolled_window(void);
 public:
-  friend void HelpDialogCB::help_button_clicked(GtkWidget*, void*);
-  friend gboolean HelpDialogCB::help_key_press_event(GtkWidget*, GdkEventKey*, void*);
+  friend void HelpDialogCB::help_dialog_button_clicked(GtkWidget*, void*);
+  friend gboolean HelpDialogCB::help_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
 
   static int get_is_helpfile(void) {return is_helpfile;}
   HelpDialog(const int size);
