@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 to 2010 Chris Vine
+/* Copyright (C) 2005 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -88,10 +88,18 @@ namespace Utf8 {
 class ConversionError: public std::exception {
   gchar* message;
 public:
-  virtual const char* what() const throw() {return message;}
-  ConversionError(GError* error) throw() {
-    g_strdup_printf("Utf8::ConversionError: %s", error->message);
+  virtual const char* what() const throw() {return (const char*)message;}
+
+  ConversionError& operator=(const ConversionError& e) throw() {
+    g_free(message);
+    message = g_strdup(e.message);
+    return *this;
   }
+  ConversionError(const ConversionError& e) throw(): message(g_strdup(e.message)) {}
+  ConversionError(const char* msg) throw():
+    message(g_strdup_printf("Utf8::ConversionError: %s", msg)) {}
+  ConversionError(GError* error) throw():
+    message(g_strdup_printf("Utf8::ConversionError: %s", error->message)) {}
   ~ConversionError() throw() {g_free(message);}
 };
 
