@@ -162,8 +162,8 @@ Callback::Functor EmitterArg<void>::connect(const Callback::Functor& f1, Release
   // f3 is the functor the remote object executes in its Releaser if it is destroyed
   // before we are, or if Releaser::operator=() is called
 
-  Callback::SafeFunctor f3(Callback::make(*this, &EmitterArg<void>::tracking_disconnect, f1));
-  Callback::Functor f2(Callback::make(r, &Releaser::remove, f3));
+  Callback::SafeFunctor f3(Callback::make_val(*this, &EmitterArg<void>::tracking_disconnect, f1));
+  Callback::Functor f2(Callback::make_val(r, &Releaser::remove, f3));
   r.add(f3);
   try {
     emission_list.push_back(ListItem(f1, f2));
@@ -389,8 +389,8 @@ Callback::SafeFunctor SafeEmitterArg<void>::connect(const Callback::SafeFunctor&
   // f3 is the functor the remote object executes in its Releaser if it is destroyed
   // before we are, or if Releaser::operator=() is called
 
-  Callback::SafeFunctor f3(Callback::make(*this, &SafeEmitterArg<void>::tracking_disconnect, f1));
-  Callback::SafeFunctorArg<int*> f2(Callback::make(r, &Releaser::try_remove, f3));
+  Callback::SafeFunctor f3(Callback::make_val(*this, &SafeEmitterArg<void>::tracking_disconnect, f1));
+  Callback::SafeFunctorArg<int*> f2(Callback::make_val(r, &Releaser::try_remove, f3));
   // we can't call Releaser::add() when holding our mutex or we will
   // get out of order locking, as Releaser's mutex is held when it
   // calls SafeEmitterArg<FreeArg>::tracking_disconnect() via f3, and
