@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 to 2010 Chris Vine
+/* Copyright (C) 2005 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -40,6 +40,7 @@ of the source code in this file).
 #define CGU_GOBJ_HANDLE_H
 
 #include <exception>
+#include <functional> // for std::less
 
 #include <glib-object.h>
 
@@ -105,7 +106,7 @@ namespace Cgu {
  * pointer), or by explicitly calling the get() method to obtain the
  * underlying pointer.
  *
- * The principle intended usage of GobjHandle is to automatically
+ * The principal intended usage of GobjHandle is to automatically
  * handle GObject reference counts and therefore to make GObjects
  * exception-safe, but they also permit GObjects/GtkWidgets to be kept
  * in standard C++ containers.
@@ -118,6 +119,14 @@ namespace Cgu {
  * than the reference count - say when accessing the referenced object
  * itself in different threads.
  *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for GobjHandles, but only if the library is compiled with
+ * the --with-smart-ptr-comp option, or if the user code defines the
+ * symbol CGU_USE_SMART_PTR_COMPARISON before gobj_handle.h is first
+ * parsed.  This is because, if user code has provided such operators
+ * for these smart pointers itself, a duplicated function definition
+ * would arise.
+ *
  * Typical usage might be, for example, as follows:
  *
  * @code
@@ -241,7 +250,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled GObject.
+  * @return A pointer to the handled GObject (or NULL if none is
+  * handled).
   */
   T* get() const {return obj_p;}
 
@@ -253,13 +263,15 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled GObject.
+  * @return A pointer to the handled GObject (or NULL if none is
+  * handled).
   */
   T* operator->() const {return obj_p;}
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled GObject.
+  * @return A pointer to the handled GObject (or NULL if none is
+  * handled).
   */
   operator T*() const {return obj_p;}
 
@@ -549,6 +561,65 @@ public:
 							     (void**)&obj_p);}
 };
 
+#if defined(CGU_USE_SMART_PTR_COMPARISON) || defined(DOXYGEN_PARSING)
+
+// we can use built-in operator == when comparing pointers referencing
+// different objects of the same type
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before gobj_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator==(const GobjHandle<T>& h1, const GobjHandle<T>& h2) {
+  return (h1.get() == h2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before gobj_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator!=(const GobjHandle<T>& h1, const GobjHandle<T>& h2) {
+  return !(h1 == h2);
+}
+
+// we must use std::less rather than the < built-in operator for
+// pointers to objects not within the same array or object: "For
+// templates greater, less, greater_equal, and less_equal, the
+// specializations for any pointer type yield a total order, even if
+// the built-in operators <, >, <=, >= do not." (para 20.3.3/8).
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before gobj_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator<(const GobjHandle<T>& h1, const GobjHandle<T>& h2) {
+  return std::less<T*>()(h1.get(), h2.get());
+}
+
+#endif // CGU_USE_SMART_PTR_COMPARISON
+
 } // namespace Cgu
 
 #endif
