@@ -1,4 +1,4 @@
-/* Copyright (C) 2006 and 2009 Chris Vine
+/* Copyright (C) 2006 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -45,6 +45,8 @@ of the source code in this file).
 // definition in the user code)
 /* #define CGU_INTRUSIVE_LOCK_COUNTER_USE_MUTEX 1 */
 
+#include <functional> // for std::less
+
 #ifdef CGU_INTRUSIVE_LOCK_COUNTER_USE_MUTEX
 #include <c++-gtk-utils/mutex.h>
 #else
@@ -90,6 +92,14 @@ namespace Cgu {
  * with IntrusivePtr is that an already-managed object may safely be
  * passed to the constructor taking a raw pointer without any
  * additional steps being necessary.)
+ *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for IntrusivePtr, but only if the library is compiled
+ * with the --with-smart-ptr-comp option, or if the user code defines
+ * the symbol CGU_USE_SMART_PTR_COMPARISON before intrusive_ptr.h is
+ * first parsed.  This is because, if user code has provided such
+ * operators for these smart pointers itself, a duplicated function
+ * definition would arise.
  */
 
 template <class T> class IntrusivePtr {
@@ -177,7 +187,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* get() const {return obj_p;}
 
@@ -189,7 +200,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* operator->() const {return obj_p;}
 
@@ -364,6 +376,65 @@ public:
   virtual ~IntrusiveLockCounter() {}
 };
 
+#if defined(CGU_USE_SMART_PTR_COMPARISON) || defined(DOXYGEN_PARSING)
+
+// we can use built-in operator == when comparing pointers referencing
+// different objects of the same type
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before intrusive_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator==(const IntrusivePtr<T>& s1, const IntrusivePtr<T>& s2) {
+  return (s1.get() == s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before intrusive_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator!=(const IntrusivePtr<T>& s1, const IntrusivePtr<T>& s2) {
+  return !(s1 == s2);
+}
+
+// we must use std::less rather than the < built-in operator for
+// pointers to objects not within the same array or object: "For
+// templates greater, less, greater_equal, and less_equal, the
+// specializations for any pointer type yield a total order, even if
+// the built-in operators <, >, <=, >= do not." (para 20.3.3/8).
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before intrusive_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator<(const IntrusivePtr<T>& s1, const IntrusivePtr<T>& s2) {
+  return std::less<T*>()(s1.get(), s2.get());
+}
+
+#endif // CGU_USE_SMART_PTR_COMPARISON
+
 } // namespace Cgu
 
 #endif
