@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 to 2010 Chris Vine
+/* Copyright (C) 2004 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -47,6 +47,7 @@ of the source code in this file).
 
 #include <exception>
 #include <new>
+#include <functional> // for std::less
 #include <cstdlib>
 
 #include <glib.h>
@@ -148,6 +149,14 @@ namespace Cgu {
  * default, and some typedef'ed instances of SharedHandle for gchar
  * (with the GFree deleter) and for GError (with the GerrorFree
  * deleter) are provided.
+ *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for SharedHandles, but only if the library is compiled
+ * with the --with-smart-ptr-comp option, or if the user code defines
+ * the symbol CGU_USE_SMART_PTR_COMPARISON before shared_handle.h is
+ * first parsed.  This is because, if user code has provided such
+ * operators for these smart pointers itself, a duplicated function
+ * definition would arise.
  */
 
 /********************* here are some deleter classes *******************/
@@ -756,16 +765,17 @@ public:
     return *this;
   }
 
-  
- /**
+   /**
   * This method does not throw.
-  * @return A pointer to the handled object.
+  * @return A pointer to the handled object (or NULL if none is
+  * handled).
   */
   T get() const {return ref_items.obj;}
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled object.
+  * @return A pointer to the handled object (or NULL if none is
+  * handled).
   */
   operator T() const {return ref_items.obj;}
 
@@ -796,7 +806,7 @@ public:
  * as the argument to a copy constructor and therefore which cannot
  * have a reference count of more than 1. It is used where, if you
  * wanted pointer semantics, you might use a const std::auto_ptr<>,
- * except that it has a reset() method..
+ * except that it has a reset() method.
  *
  * As of version 1.0.2, ScopedHandle objects can be instantiated for
  * pointers to constant objects (such as ScopedHandle<const char*>),
@@ -853,7 +863,8 @@ public:
 /**
  * Causes the ScopedHandle to cease to manage the handled object, but
  * does not delete that object.  This method does not throw.
- * @return A pointer to the previously handled object.
+ * @return A pointer to the previously handled object (or NULL if none
+ * was handled).
  *
  * Since 1.0.2
  */
@@ -861,13 +872,15 @@ public:
 
 /**
  * This method does not throw.
- * @return A pointer to the handled object.
+ * @return A pointer to the handled object (or NULL if none is
+ * handled).
  */
   T get() const {return obj;}
 
 /**
  * This method does not throw.
- * @return A pointer to the handled object.
+ * @return A pointer to the handled object (or NULL if none is
+ * handled).
  */
   operator T() const {return obj;}
 
@@ -900,10 +913,10 @@ public:
  * containers which exist in different threads but which contain
  * shared objects by reference.  But:
  *
- *  1.  If the referenced object is to be modified in one thread and
- *      read or modified in another thread an appropriate mutex for
- *      the referenced object is required (unless that referenced
- *      object does its own locking).
+ * 1.  If the referenced object is to be modified in one thread and
+ *     read or modified in another thread an appropriate mutex for the
+ *     referenced object is required (unless that referenced object
+ *     does its own locking).
  *
  * 2.  If the same instance of shared handle is to be modified in one
  *     thread (by assigning to the handle so that it references a
@@ -939,6 +952,14 @@ public:
  * CGU_SHARED_LOCK_HANDLE_USE_MUTEX before shared_handle.h is parsed
  * so as to use mutexes instead, which might be useful for some
  * debugging purposes.
+ *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for SharedLockHandles, but only if the library is
+ * compiled with the --with-smart-ptr-comp option, or if the user code
+ * defines the symbol CGU_USE_SMART_PTR_COMPARISON before
+ * shared_handle.h is first parsed.  This is because, if user code has
+ * provided such operators for these smart pointers itself, a
+ * duplicated function definition would arise.
  */
 
 template <class T, class Dealloc = StandardArrayDelete<T> > class SharedLockHandle {
@@ -1434,13 +1455,15 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled object.
+  * @return A pointer to the handled object (or NULL if none is
+  * handled).
   */
   T get() const {return ref_items.obj;}
 
  /**
   * This method does not throw.
-  * @return A pointer to the handled object.
+  * @return A pointer to the handled object (or NULL if none is
+  * handled).
   */
   operator T() const {return ref_items.obj;}
 
@@ -1470,6 +1493,113 @@ public:
   ~SharedLockHandle() {unreference();}
 };
 
+#if defined(CGU_USE_SMART_PTR_COMPARISON) || defined(DOXYGEN_PARSING)
+
+// we can use built-in operator == when comparing pointers referencing
+// different objects of the same type
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator==(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) {
+  return (s1.get() == s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator!=(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) {
+  return !(s1 == s2);
+}
+
+// we must use std::less rather than the < built-in operator for
+// pointers to objects not within the same array or object: "For
+// templates greater, less, greater_equal, and less_equal, the
+// specializations for any pointer type yield a total order, even if
+// the built-in operators <, >, <=, >= do not." (para 20.3.3/8).
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator<(const SharedHandle<T, Dealloc>& s1, const SharedHandle<T, Dealloc>& s2) {
+  return std::less<T>()(s1.get(), s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator==(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) {
+  return (s1.get() == s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator!=(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) {
+  return !(s1 == s2);
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_handle.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T, class Dealloc>
+bool operator<(const SharedLockHandle<T, Dealloc>& s1, const SharedLockHandle<T, Dealloc>& s2) {
+  return std::less<T>()(s1.get(), s2.get());
+}
+
+#endif // CGU_USE_SMART_PTR_COMPARISON
+
 } // namespace Cgu
 
 #endif
