@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 to 2010 Chris Vine
+/* Copyright (C) 2004 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -47,6 +47,7 @@ of the source code in this file).
 
 #include <exception>
 #include <new>
+#include <functional> // for std::less
 
 #ifdef CGU_SHARED_LOCK_PTR_USE_MUTEX
 #include <c++-gtk-utils/mutex.h>
@@ -139,6 +140,14 @@ namespace SharedPtrAllocFail {
  * allocated on freestore with the new expression.  A managed object
  * will be deleted when the last SharedPtr referencing it is
  * destroyed.
+ *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for SharedPtr, but only if the library is compiled with
+ * the --with-smart-ptr-comp option, or if the user code defines the
+ * symbol CGU_USE_SMART_PTR_COMPARISON before shared_ptr.h is first
+ * parsed.  This is because, if user code has provided such operators
+ * for these smart pointers itself, a duplicated function definition
+ * would arise.
  */
 
 template <class T> class SharedPtr {
@@ -439,7 +448,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* get() const {return ref_items.obj_p;}
 
@@ -451,7 +461,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* operator->() const {return ref_items.obj_p;}
 
@@ -472,7 +483,7 @@ public:
 /**
  * @class SharedLockPtr shared_ptr.h c++-gtk-utils/shared_ptr.h
  * @brief This is a smart pointer for managing the lifetime of objects
- * allocated on freestore, with a thread safe reference count..
+ * allocated on freestore, with a thread safe reference count.
  * @ingroup handles
  * @sa SharedPtr SharedPtrError
  *
@@ -495,10 +506,10 @@ public:
  *     does its own locking).
  *
  * 2.  If the same instance of shared pointer is to be modified in one
- *    thread (by assigning to the pointer so that it references a
- *    different object), and copied (assigned from or used as the
- *    argument of a copy constructor) or modified in another thread, a
- *    mutex for that instance of shared pointer is required.
+ *     thread (by assigning to the pointer so that it references a
+ *     different object), and copied (assigned from or used as the
+ *     argument of a copy constructor) or modified in another thread,
+ *     a mutex for that instance of shared pointer is required.
  *
  * 3.  Objects referenced by shared pointers which are objects for
  *     which POSIX provides no guarantees (in the main, those which
@@ -517,6 +528,14 @@ public:
  * CGU_SHARED_LOCK_PTR_USE_MUTEX before shared_ptr.h is parsed so as
  * to use mutexes instead, which might be useful for some debugging
  * purposes.
+ *
+ * From version 1.2.12, the library provides ==, != and < comparison
+ * operators for SharedLockPtr, but only if the library is compiled
+ * with the --with-smart-ptr-comp option, or if the user code defines
+ * the symbol CGU_USE_SMART_PTR_COMPARISON before shared_ptr.h is
+ * first parsed.  This is because, if user code has provided such
+ * operators for these smart pointers itself, a duplicated function
+ * definition would arise.
  */
 
 template <class T> class SharedLockPtr {
@@ -1032,7 +1051,8 @@ public:
   
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* get() const {return ref_items.obj_p;}
 
@@ -1044,7 +1064,8 @@ public:
 
  /**
   * This method does not throw.
-  * @return A pointer to the managed object.
+  * @return A pointer to the managed object (or NULL if none is
+  * managed).
   */
   T* operator->() const {return ref_items.obj_p;}
 
@@ -1074,6 +1095,113 @@ public:
   ~SharedLockPtr() {unreference();}
 };
 
+#if defined(CGU_USE_SMART_PTR_COMPARISON) || defined(DOXYGEN_PARSING)
+
+// we can use built-in operator == when comparing pointers referencing
+// different objects of the same type
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator==(const SharedPtr<T>& s1, const SharedPtr<T>& s2) {
+  return (s1.get() == s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator!=(const SharedPtr<T>& s1, const SharedPtr<T>& s2) {
+  return !(s1 == s2);
+}
+
+// we must use std::less rather than the < built-in operator for
+// pointers to objects not within the same array or object: "For
+// templates greater, less, greater_equal, and less_equal, the
+// specializations for any pointer type yield a total order, even if
+// the built-in operators <, >, <=, >= do not." (para 20.3.3/8).
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator<(const SharedPtr<T>& s1, const SharedPtr<T>& s2) {
+  return std::less<T*>()(s1.get(), s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator==(const SharedLockPtr<T>& s1, const SharedLockPtr<T>& s2) {
+  return (s1.get() == s2.get());
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator!=(const SharedLockPtr<T>& s1, const SharedLockPtr<T>& s2) {
+  return !(s1 == s2);
+}
+
+/**
+ * @ingroup handles
+ *
+ * This comparison operator does not throw.  It compares the addresses
+ * of the managed objects.  This function is only available if the
+ * library is compiled with the --with-smart-ptr-comp option, or if
+ * the user code defines the symbol CGU_USE_SMART_PTR_COMPARISON
+ * before shared_ptr.h is first parsed.
+ *
+ * Since 1.2.12
+ */
+template <class T>
+bool operator<(const SharedLockPtr<T>& s1, const SharedLockPtr<T>& s2) {
+  return std::less<T*>()(s1.get(), s2.get());
+}
+
+#endif // CGU_USE_SMART_PTR_COMPARISON
+
 } // namespace Cgu
 
 #endif
