@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 to 2010 Chris Vine
+/* Copyright (C) 2005 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -24,7 +24,11 @@ License as follows:
 
 #include <c++-gtk-utils/lib_defs.h>
 
+#include <glib.h>
+
 #include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/application.h>
+
 
 #ifndef DOXYGEN_PARSING
 namespace Cgu {
@@ -55,7 +59,18 @@ namespace Cgu {
 WinBase::WinBase(const char* caption, GdkPixbuf* icon_p, bool modal,
 		 GtkWindow* parent_p_, GtkWindow* window_p):
 			     in_exec_loop(false), is_modal(modal),
-			     close_guard(false), parent_p(parent_p_) {
+			     close_guard(false) {
+
+  // use memory slices here: they will not throw (so maintaining API
+  // compatibility with versions prior to 1.2.12) and since a number
+  // of the following GTK+ functions will invoke
+  // g_slice_new()/g_slice_alloc(), we are not losing anything by
+  // doing so in terms of program robustness
+  impl = g_slice_new(Impl);
+  impl->parent_p = parent_p_;
+#if GTK_CHECK_VERSION(2,99,0)
+  impl->app_p = 0;
+#endif
 
   if (window_p) g_window_p = window_p;
   else {
@@ -66,9 +81,9 @@ WinBase::WinBase(const char* caption, GdkPixbuf* icon_p, bool modal,
 
   if (is_modal) {
     gtk_window_set_modal(g_window_p, true);
-    if (parent_p) {
-      gtk_window_set_transient_for(g_window_p, parent_p);
-      gtk_widget_set_sensitive(GTK_WIDGET(parent_p), false);
+    if (parent_p_) {
+      gtk_window_set_transient_for(g_window_p, parent_p_);
+      gtk_widget_set_sensitive(GTK_WIDGET(parent_p_), false);
     }
   }
 
@@ -79,6 +94,10 @@ WinBase::WinBase(const char* caption, GdkPixbuf* icon_p, bool modal,
 }
 
 WinBase::~WinBase() {
+#if GTK_CHECK_VERSION(2,99,0)
+  if (impl->app_p) impl->app_p->remove(this);
+#endif
+
   gtk_widget_destroy(GTK_WIDGET(g_window_p));
   // call gtk_main_quit() if we have previously called WinBase::exec() on this
   // object and we have not reached this destructor via a call to WinBase::close()
@@ -88,6 +107,8 @@ WinBase::~WinBase() {
   // gtk_main_quit() will balance up the recursive calls to gtk_main() to ensure
   // that both the parent and this object are unblocked and so destroyed)
   if (in_exec_loop) gtk_main_quit();
+
+  g_slice_free(Impl, impl);
 }
 
 int WinBase::get_exec_val() const {
@@ -107,8 +128,11 @@ void WinBase::close() {
   if (close_guard) return;
   close_guard = true;
 
-  if (is_modal && parent_p) gtk_widget_set_sensitive(GTK_WIDGET(parent_p), true);
+  if (is_modal && impl->parent_p)
+    gtk_widget_set_sensitive(GTK_WIDGET(impl->parent_p), true);
+
   gtk_widget_hide(GTK_WIDGET(g_window_p));
+
   if (in_exec_loop) {
     in_exec_loop = false;
     gtk_main_quit();
@@ -123,6 +147,14 @@ void WinBase::on_delete_event() {
 }
 
 int WinBase::exec() {
+#if GTK_CHECK_VERSION(2,99,0)
+  if (impl->app_p) {
+    g_critical("Cgu::WinBase::exec() called in relation to a WinBase "
+	       "object added to a Cgu::Application object\n");
+    return -1;
+  }
+#endif
+
   in_exec_loop = true;
   gtk_main();
   return get_exec_val();
