@@ -1,4 +1,4 @@
-/* Copyright (C) 2005 to 2010 Chris Vine
+/* Copyright (C) 2005 to 2011 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -33,7 +33,7 @@ License as follows:
  * @class Cgu::WinBase window.h c++-gtk-utils/window.h
  * @brief This is a class for managing the lifetime of top level
  * widgets.
- * @sa MainWidgetBase
+ * @sa MainWidgetBase Application
  *
  * This class provides a base class for lifetime management of top
  * level widgets which can make GTK+ exception safe.  It would be
@@ -102,7 +102,10 @@ License as follows:
  * the constructor of a derived class may want to do this which
  * requires the widget not to be visible.  Accordingly the constructor
  * of the derived class (or the library user) should call
- * gtk_widget_show()/gtk_widget_show_all() via WinBase::get_win()) .
+ * gtk_widget_show()/gtk_widget_show_all() via WinBase::get_win()).
+ *
+ * See @ref Threading for particulars about GTK+ thread safety (and so
+ * WinBase thread safety).
  *
  * A small compilable example is as follows:
  *
@@ -201,14 +204,35 @@ License as follows:
 
 namespace Cgu {
 
+#if GTK_CHECK_VERSION(2,99,0)
+class Application;
+#endif
+
 class WinBase {
+
   // main class object
   GtkWindow* g_window_p;
 
   bool in_exec_loop;
   bool is_modal;
   bool close_guard;
+
+  // we only use this struct to retain binary compatibility with
+  // 1.2 series versions before 1.2.12 (we have changed the parent_p
+  // pointer to the impl pointer).
+struct Impl {
   GtkWindow* parent_p;
+#if GTK_CHECK_VERSION(2,99,0)
+  Application* app_p;
+#endif
+};
+  Impl* impl;
+
+#if GTK_CHECK_VERSION(2,99,0)
+  // only Cgu::Application can access these
+  void set_application(Application* app) {impl->app_p = app;}
+  void unset_application() {impl->app_p = 0;}
+#endif
 
   // WinBase cannot be copied
   WinBase(const WinBase&);
@@ -241,6 +265,9 @@ protected:
    */
   virtual void on_delete_event();
 public:
+#if GTK_CHECK_VERSION(2,99,0)
+  friend class Application;
+#endif
 #ifndef DOXYGEN_PARSING
   // this helper class avoids exposing GObject callbacks with C
   // linkage to the global namespace
@@ -258,7 +285,10 @@ public:
    * Makes the window block. Calls gtk_main().  It is usually a bad
    * idea to call this method (and so have nested loops) on a
    * non-modal dialog.  This method will not throw.
-   * @return The value returned by get_exec_val().
+   * @return The value returned by get_exec_val()
+   * @note If this library is compiled against GTK+3 and the WinBase
+   * object has been added to a Cgu::Application object, then this
+   * method returns immediately with a value of -1.
    */
   int exec();
 
@@ -302,7 +332,10 @@ public:
 	  GtkWindow* parent = 0, GtkWindow* window = 0);
 
   /**
-   * The destructor will not throw.
+   * The destructor will not throw assuming (in a case where the
+   * Cgu::WinBase object has been added to a Cgu::Application object)
+   * that merely iterating through a list does not throw (as it would
+   * not on any sane implementation).
    */
   virtual ~WinBase();
 
