@@ -49,15 +49,12 @@ public:
   ~Logger(void);
 };
 
-namespace { // we put the functions in anonymous namespace in helpfile.cpp
-            // so they are not exported at link time
 namespace LogViewDialogCB {
   extern "C" {
-    void log_view_button_clicked(GtkWidget*, void*);
-    gboolean log_view_key_press_event(GtkWidget*, GdkEventKey*, void*);
+    void log_view_dialog_button_clicked(GtkWidget*, void*);
+    gboolean log_view_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
   }
 }
-}   // anonymous namespace
 
 struct LogViewFileError: public std::exception {
   virtual const char* what() const throw() {return "Cannot open log file for viewing";}
@@ -66,8 +63,8 @@ struct LogViewFileError: public std::exception {
 class LogViewDialog: public WinBase {
   GtkWidget* text_view_p;
 public:
-  friend void LogViewDialogCB::log_view_button_clicked(GtkWidget*, void*);
-  friend gboolean LogViewDialogCB::log_view_key_press_event(GtkWidget*, GdkEventKey*, void*);
+  friend void LogViewDialogCB::log_view_dialog_button_clicked(GtkWidget*, void*);
+  friend gboolean LogViewDialogCB::log_view_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
   LogViewDialog(const int standard_size, const std::string& logfile_name, GtkWindow* parent_p = 0);
 };
 
