@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -136,7 +136,7 @@ int main(int argc, char* argv[]) {
   if (argc > 1) {
     if (is_arg("--version", argc, argv)) {
       std::string message("efax-gtk-" VERSION "\n");
-      message += gettext("Copyright (C) 2001 - 2010 Chris Vine\n"
+      message += gettext("Copyright (C) 2001 - 2011 Chris Vine\n"
 			 "This program is released under the "
 			 "GNU General Public License, version 2\n");
       try {
@@ -411,6 +411,7 @@ std::string configure_prog(bool reread) {
     std::string fax_received_exec;
     std::string addr_in_header;
     std::string redial;
+    std::string ip_family;
     std::string work_subdir;
     
     while (std::getline(filein, file_read)) {
@@ -493,6 +494,9 @@ std::string configure_prog(bool reread) {
 	// look for "SOCK_OTHER_ADDRESSES:"
 	else if (get_prog_parm("SOCK_OTHER_ADDRESSES:", file_read, sock_other_addresses));
 
+	// look for "IP_FAMILY:"
+	else if (get_prog_parm("IP_FAMILY:", file_read, ip_family));
+
 	// look for "FAX_RECEIVED_POPUP:"
 	else if (get_prog_parm("FAX_RECEIVED_POPUP:", file_read, fax_received_popup));
 
@@ -811,6 +815,14 @@ std::string configure_prog(bool reread) {
       }
     }
 
+    temp = utf8_to_lower(ip_family);
+    if (temp.empty() || !temp.compare("ipv4")) prog_config.ipv6 = false;
+    else if (!temp.compare("ipv6")) prog_config.ipv6 = true;
+    else {
+      return_val += gettext("Invalid ip family for socket server specified: adopting default of ipv4\n");
+      prog_config.ipv6 = false;
+    }
+
     temp = utf8_to_lower(fax_received_popup);
     if (!temp.compare("yes")) prog_config.fax_received_popup = true;
     else prog_config.fax_received_popup = false;
