@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -36,7 +36,7 @@ COPYING distributed with the source files.
 #include "socket_notify.h"
 #include "menu_icons.h"
 #include "utils/icon_info_handle.h"
-#if !(GTK_CHECK_VERSION(2,91,0))
+#if !(GTK_CHECK_VERSION(2,99,0))
 #include "utils/cairo_handle.h"
 #endif
 
@@ -181,7 +181,7 @@ void MainWindowCB::mainwin_menuitem_activated(GtkMenuItem* item_p, void* data) {
 gboolean MainWindowCB::mainwin_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   MainWindow* instance_p = static_cast<MainWindow*>(data);
   
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   if (event_p->keyval == GDK_KEY_F1) {
     instance_p->helpfile_impl();
     return true; // processing stops here
@@ -217,23 +217,32 @@ gboolean MainWindowCB::mainwin_window_state_event(GtkWidget*, GdkEventWindowStat
   return false;  // carry on processing the window state event
 }
 
-void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle* prev_style_p, void* data) {
-  static_cast<MainWindow*>(data)->style_set_impl(prev_style_p);
+#if GTK_CHECK_VERSION(2,99,0)
+
+void MainWindowCB::mainwin_button_style_updated(GtkWidget*, void* data) {
+  static_cast<MainWindow*>(data)->button_style_updated_impl();
 }
 
-#if GTK_CHECK_VERSION(2,91,0)
 gboolean MainWindowCB::mainwin_drawing_area_draw(GtkWidget*,
 						 cairo_t* cairo_p,
 						 void* data) {
   return static_cast<MainWindow*>(data)->draw_fax_from_socket_notifier(cairo_p);
 }
+
 #else
+
+void MainWindowCB::mainwin_button_style_set(GtkWidget*, GtkStyle* prev_style_p, void* data) {
+  static_cast<MainWindow*>(data)->button_style_set_impl(prev_style_p);
+}
+
 gboolean MainWindowCB::mainwin_drawing_area_expose_event(GtkWidget*,
 							 GdkEventExpose* event_p,
 							 void* data) {
   return static_cast<MainWindow*>(data)->draw_fax_from_socket_notifier(event_p);
 }
-#endif
+
+#endif /* GTK_CHECK_VERSION(2,99,0) */
+
 
 MainWindow::MainWindow(const std::string& messages,bool start_hidden,
 		       bool start_in_standby, const char* filename):
@@ -711,8 +720,14 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
 		   G_CALLBACK(MainWindowCB::mainwin_visibility_notify_event), this);
   g_signal_connect(G_OBJECT(get_win()), "window_state_event",
 		   G_CALLBACK(MainWindowCB::mainwin_window_state_event), this);
-  g_signal_connect(G_OBJECT(get_win()), "style_set",
-		   G_CALLBACK(MainWindowCB::mainwin_style_set), this);
+  // any button will do respond to a button style change (we use the 'Stop' button)
+#if GTK_CHECK_VERSION(2,99,0)
+  g_signal_connect(G_OBJECT(stop_button_p), "style_updated",
+		   G_CALLBACK(MainWindowCB::mainwin_button_style_updated), this);
+#else
+  g_signal_connect(G_OBJECT(stop_button_p), "style_set",
+                   G_CALLBACK(MainWindowCB::mainwin_button_style_set), this);
+#endif
 
   gtk_container_set_border_width(GTK_CONTAINER(win_table_p), standard_size/3);
 
@@ -729,7 +744,7 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
   gtk_container_set_focus_chain(GTK_CONTAINER(win_table_p), focus_chain_p);
   g_list_free(focus_chain_p);
 
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   g_signal_connect(G_OBJECT(drawing_area_p), "draw",
 		   G_CALLBACK(MainWindowCB::mainwin_drawing_area_draw),
 		   this);
@@ -842,7 +857,8 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
   // start the socket server
   if (prog_config.sock_server && !prog_config.sock_server_port.empty()) {
     socket_server.start(std::string(prog_config.sock_server_port),
-			prog_config.other_sock_client_address);
+			prog_config.other_sock_client_address,
+			prog_config.ipv6);
   }
 
   tray_icon.activated.connect(Callback::make(*this,
@@ -1473,12 +1489,11 @@ void MainWindow::settings_impl(void) {
   efax_controller.get_state(state);
 
   if (state != EfaxController::inactive) {
-    InfoDialog* dialog_p;
     std::string message(gettext("Can't change settings unless "
 				"the program is inactive\n"
 				"Press the Stop button to make it inactive"));
-    dialog_p = new InfoDialog(message.c_str(), "Change settings",
-			      GTK_MESSAGE_WARNING, get_win());
+    new InfoDialog(message.c_str(), "Change settings",
+		   GTK_MESSAGE_WARNING, get_win());
 
     // there is no memory leak -- exec() was not called so InfoDialog
     // will delete its own memory when it is closed
@@ -1512,13 +1527,16 @@ void MainWindow::settings_changed_cb(const std::string& messages) {
       && !prog_config.sock_server_port.empty()
       && !socket_server.is_server_running()) {
     socket_server.start(std::string(prog_config.sock_server_port),
-			prog_config.other_sock_client_address);
+			prog_config.other_sock_client_address,
+			prog_config.ipv6);
   }
   else if (socket_server.get_port() != std::string(prog_config.sock_server_port)
-	   || socket_server.get_other_sock_client_address() != prog_config.other_sock_client_address) {
+	   || socket_server.get_other_sock_client_address() != prog_config.other_sock_client_address
+	   || socket_server.is_ipv6() != prog_config.ipv6) {
     socket_server.stop();
     socket_server.start(std::string(prog_config.sock_server_port),
-			prog_config.other_sock_client_address);
+			prog_config.other_sock_client_address,
+			prog_config.ipv6);
   }
 
   if (prog_config.logfile_name.empty()) {
@@ -1612,17 +1630,15 @@ std::pair<int, int> MainWindow::get_max_button_text_extents(void) {
   int max_width;
   int max_height;
 
-  // create a button to work on - the GobjHandle object will claim ownership of the button
-  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
-  GobjHandle<GtkWidget> button_h(gtk_button_new());
-  gtk_widget_set_style(button_h, style_p);
-
   for (iter = text_vec.begin(), width = 0, height = 0, max_width = 0, max_height = 0;
        iter != text_vec.end();
        ++iter) {
 
-    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(button_h,
+    // any button will do to get a pango layout (we have connected
+    // the relevant style signal to the 'Stop' button)
+    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(stop_button_p,
 								    iter->c_str()));
+
     pango_layout_get_pixel_size(layout_h, &width, &height);
 
     if (width > max_width) {
@@ -1635,34 +1651,6 @@ std::pair<int, int> MainWindow::get_max_button_text_extents(void) {
   return std::pair<int, int>(max_width, max_height);
 }
 
-int MainWindow::get_max_status_text_height(void) {
-  std::vector<std::string> text_vec(efax_controller.get_state_messages());
-  text_vec.push_back(gettext("Press F1 for help"));
-  std::vector<std::string>::const_iterator iter;
-  int width;
-  int height;
-  int max_height;
-
-  // create a label to work on - the GobjHandle object will claim ownership of the label
-  GtkStyle* style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
-  GobjHandle<GtkWidget> label_h(gtk_label_new(0));
-  gtk_widget_set_style(label_h, style_p);
-
-  for (iter = text_vec.begin(), width = 0, height = 0, max_height = 0;
-       iter != text_vec.end();
-       ++iter) {
-
-    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(label_h,
-								    iter->c_str()));
-    pango_layout_get_pixel_size(layout_h, &width, &height);
-
-    if (height > max_height) {
-      max_height = height;
-    }
-  }
-  return max_height;
-}
-
 void MainWindow::tray_icon_activated_cb(void) {
 
   GtkWidget* mainwin_p = GTK_WIDGET(get_win());
@@ -1720,46 +1708,66 @@ void MainWindow::tray_icon_menu_cb(int item) {
   }
 }
 
-void MainWindow::style_set_impl(GtkStyle* prev_style_p) {
+void MainWindow::button_style_updated_impl(void) {
+
+  std::pair<int, int> extents = get_max_button_text_extents();
+
+  // add padding
+  extents.first += 18;  // width
+  extents.second += 12; // height
+
+  // have some sensible minimum width and height if a very small font chosen
+  const int button_min_width = standard_size * 4;
+  const int button_min_height = 30;
+  if (extents.first < button_min_width) extents.first = button_min_width;
+  if (extents.second < button_min_height) extents.second = button_min_height;
+
+  gtk_widget_set_size_request(single_file_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(multiple_file_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(socket_list_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(number_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(send_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(receive_answer_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(receive_takeover_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(receive_standby_button_p, extents.first, extents.second);
+  gtk_widget_set_size_request(stop_button_p, extents.first, extents.second);
+
+  SharedPtr<std::vector<std::string> > text_vec(new std::vector<std::string>(efax_controller.get_state_messages()));
+  text_vec->push_back(gettext("Press F1 for help"));
+
+#if GTK_CHECK_VERSION(2,20,0)
+  if (!gtk_widget_get_realized(status_line.get_main_widget())) {
+    status_line.set_status_line(text_vec);
+  }
+#else
+  if (!GTK_WIDGET_REALIZED(status_line.get_main_widget())) {
+    status_line.set_status_line(text_vec);
+  }
+#endif
+
+  else {
+    // invoke StatusLine::set_status_line() through a Callback object executed
+    // via the main loop with G_PRIORITY_DEFAULT_IDLE if we are not in program
+    // start-up, so that we know the status line has received its new style when
+    // the function is executed
+    Callback::post(Callback::make_val(status_line, &StatusLine::set_status_line, text_vec));
+  }
+}
+
+#if !(GTK_CHECK_VERSION(2,99,0))
+void MainWindow::button_style_set_impl(GtkStyle* prev_style_p) {
 
   GtkStyle* new_style_p = gtk_widget_get_style(GTK_WIDGET(get_win()));
   // we only need to proceed further if the font has changed
   if (!prev_style_p || !pango_font_description_equal(prev_style_p->font_desc,
 						     new_style_p->font_desc)) {
 
-    std::pair<int, int> extents = get_max_button_text_extents();
-
-    // add padding
-    extents.first += 18;  // width
-    extents.second += 12; // height
-
-    // have some sensible minimum width and height if a very small font chosen
-    const int button_min_width = standard_size * 4;
-    const int button_min_height = 30;
-    if (extents.first < button_min_width) extents.first = button_min_width;
-    if (extents.second < button_min_height) extents.second = button_min_height;
-
-    gtk_widget_set_size_request(single_file_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(multiple_file_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(socket_list_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(number_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(send_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(receive_answer_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(receive_takeover_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(receive_standby_button_p, extents.first, extents.second);
-    gtk_widget_set_size_request(stop_button_p, extents.first, extents.second);
-
-    int status_height = get_max_status_text_height();
-
-    // add padding
-    status_height += 2;
-
-    // set_status_line() provides a minimum height if a very small font chosen
-    status_line.set_status_line(status_height);
+    button_style_updated_impl();
   }
 }
+#endif // !(GTK_CHECK_VERSION(2,99,0))
 
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
 bool MainWindow::draw_fax_from_socket_notifier(cairo_t* cairo_p)
 #else
 bool MainWindow::draw_fax_from_socket_notifier(GdkEventExpose* event_p)
@@ -1812,7 +1820,7 @@ bool MainWindow::draw_fax_from_socket_notifier(GdkEventExpose* event_p)
      from the socket)
 */
 
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   if (gtk_widget_get_realized(drawing_area_p)) { // if we started hidden in the system tray
                                                  // the drawing area may not yet be realised
     if (!socket_server.get_filenames().first->empty()) {
@@ -1828,8 +1836,11 @@ bool MainWindow::draw_fax_from_socket_notifier(GdkEventExpose* event_p)
       cairo_stroke(cairo_p);
     }
     else {
-      gdk_cairo_set_source_color(cairo_p,
-				 &gtk_widget_get_style(drawing_area_p)->bg[gtk_widget_get_state(drawing_area_p)]);
+      GdkRGBA colour;
+      gtk_style_context_get_background_color(gtk_widget_get_style_context(drawing_area_p),
+					     gtk_widget_get_state_flags(drawing_area_p),
+					     &colour);
+      gdk_cairo_set_source_rgba(cairo_p, &colour);
       cairo_paint(cairo_p);
     }
   }
@@ -1942,7 +1953,7 @@ MessageText::MessageText(GtkWindow* win_p):
   // GtkTextTags are not owned by a GTK+ container when passed to it so use a GobjHandle
   red_tag_h = GobjHandle<GtkTextTag>(gtk_text_tag_new(0));
 
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   g_object_set(red_tag_h.get(),
 	       "foreground-gdk", &red,
 	       static_cast<void*>(0));
@@ -2117,12 +2128,47 @@ StatusLine::StatusLine(const int size):
   gtk_widget_show_all(get_main_widget());
 }
 
-void StatusLine::set_status_line(int new_height) {
+int StatusLine::get_max_status_text_height(const std::vector<std::string>& text_vec) {
+  std::vector<std::string>::const_iterator iter;
+  int width;
+  int height;
+  int max_height;
+
+  for (iter = text_vec.begin(), width = 0, height = 0, max_height = 0;
+       iter != text_vec.end();
+       ++iter) {
+
+    GobjHandle<PangoLayout> layout_h(gtk_widget_create_pango_layout(status_label_p,
+								    iter->c_str()));
+    pango_layout_get_pixel_size(layout_h, &width, &height);
+
+    if (height > max_height) {
+      max_height = height;
+    }
+  }
+  return max_height;
+}
+
+void StatusLine::set_status_line(SharedPtr<std::vector<std::string> > text_vec) {
+
+  int new_height = get_max_status_text_height(*text_vec);
+  // add padding
+  new_height += 2;
 
   // set the height
   if (new_height < standard_size) new_height = standard_size;
   gtk_widget_set_size_request(status_label_p, -1, new_height);
 
+#if GTK_CHECK_VERSION(2,99,0)
+  // make the status label display in red
+  GdkRGBA red;
+  red.red = 0.9;
+  red.green = 0.0;
+  red.blue = 0.0;
+  red.alpha = 1.0;
+
+  gtk_widget_override_color(status_label_p, GTK_STATE_FLAG_NORMAL, &red);
+#else
   // make the status label display in red
   GdkColor red;
   red.red = static_cast<guint16>(0.9 * 65535.0);
@@ -2132,6 +2178,7 @@ void StatusLine::set_status_line(int new_height) {
   // we don't need to allocate the GdkColor red object before it is
   // used by gtk_widget_modify_fg() - that function will do it itself
   gtk_widget_modify_fg(status_label_p, GTK_STATE_NORMAL, &red);
+#endif
 }
 
 void StatusLine::write_status_cb(const char* text) {
