@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -63,11 +63,12 @@ public:
 };
 
 class StatusLine: public MainWidgetBase {
-
   const int standard_size;
   GtkWidget* status_label_p;
+
+  int get_max_status_text_height(const std::vector<std::string>&);
 public:
-  void set_status_line(int);
+  void set_status_line(SharedPtr<std::vector<std::string> >);
   void write_status_cb(const char* text);
   StatusLine(const int);
 };
@@ -79,11 +80,12 @@ namespace MainWindowCB {
     gboolean mainwin_key_press_event(GtkWidget*, GdkEventKey*, void*);
     gboolean mainwin_visibility_notify_event(GtkWidget*, GdkEventVisibility*, void*);
     gboolean mainwin_window_state_event(GtkWidget*, GdkEventWindowState*, void*);
-    void mainwin_style_set(GtkWidget*, GtkStyle*, void*);
     gboolean mainwin_timer_event(void*);
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
+    void mainwin_button_style_updated(GtkWidget*, void*);
     gboolean mainwin_drawing_area_draw(GtkWidget*, cairo_t*, void*);
 #else
+    void mainwin_button_style_set(GtkWidget*, GtkStyle*, void*);
     gboolean mainwin_drawing_area_expose_event(GtkWidget*, GdkEventExpose*, void*);
 #endif
   }
@@ -167,7 +169,6 @@ class MainWindow: public WinBase {
   void helpfile_impl(void);
   void set_file_items_sensitive_impl(void);
   void set_socket_items_sensitive_impl(void);
-  void style_set_impl(GtkStyle*);
 
   // these are connected to Emitter or Notifier objects, or connected
   // to the callback of a WatchSource (iowatch) object, and might also be
@@ -190,16 +191,18 @@ class MainWindow: public WinBase {
 
   void fax_to_send_dialog(const std::pair<std::string, unsigned int>&);
   std::pair<int, int> get_max_button_text_extents(void);
-  int get_max_status_text_height(void);
   void get_window_settings(void);
   void save_window_settings(void);
   void strip(std::string&);
 
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
   bool draw_fax_from_socket_notifier(cairo_t*);
 #else
   bool draw_fax_from_socket_notifier(GdkEventExpose*);
+  void button_style_set_impl(GtkStyle*);
 #endif
+  void button_style_updated_impl(void);
+
 protected:
   virtual void on_delete_event(void);
 public:
@@ -210,11 +213,12 @@ public:
 								GdkEventVisibility*, void*);
   friend gboolean MainWindowCB::mainwin_window_state_event(GtkWidget*,
 							   GdkEventWindowState*, void*);
-  friend void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle*, void*);
-#if GTK_CHECK_VERSION(2,91,0)
+#if GTK_CHECK_VERSION(2,99,0)
+  friend void MainWindowCB::mainwin_button_style_updated(GtkWidget*, void*);
   friend gboolean MainWindowCB::mainwin_drawing_area_draw(GtkWidget*,
 							  cairo_t*, void*);
 #else
+  friend void MainWindowCB::mainwin_button_style_set(GtkWidget*, GtkStyle*, void*);
   friend gboolean MainWindowCB::mainwin_drawing_area_expose_event(GtkWidget*,
 								  GdkEventExpose*, void*);
 #endif
