@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -195,10 +195,13 @@ class SockPage: public MainWidgetBase {
   GtkWidget* other_address_button_p;
   GtkWidget* port_spin_button_p;
   GtkWidget* other_addresses_entry_p;
+  GtkWidget* ipv4_button_p;
+  GtkWidget* ipv6_button_p;
   GtkWidget* run_server_help_button_p;
   GtkWidget* popup_help_button_p;
   GtkWidget* port_help_button_p;
   GtkWidget* client_address_help_button_p;
+  GtkWidget* ip_family_help_button_p;
 
   void other_address_button_toggled_impl(void);
 public:
@@ -211,11 +214,13 @@ public:
   std::string get_port(void) const;
   std::string get_if_other_address(void) const;
   std::string get_other_addresses(void) const;
+  std::string get_ip_family(void) const;
   void set_run_server(const std::string& run_server_string);
   void set_popup(const std::string& popup_string);
   void set_port(const std::string& port_string);
   void set_if_other_address(const std::string& if_other_address_string);
   void set_other_addresses(const std::string& other_addresses_string);
+  void set_ip_family(const std::string& ip_family_string);
 
   void clear(void);
   SockPage(const int);
