@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -155,7 +155,7 @@ ViewMessages::ViewMessages(void): SettingsMessagesBase(ps_view_command + 1) {
   captions[ps_view_command] = gettext("efax-gtk help: Postscript Viewer Program");
 }
 
-SockMessages::SockMessages(void): SettingsMessagesBase(client_address + 1) {
+SockMessages::SockMessages(void): SettingsMessagesBase(ip_family + 1) {
 
   messages[run_server] = gettext("This specifies whether a socket server is to be run for CUPS or  "
 				 "some other print system.  Check the box if it is to run");
@@ -181,10 +181,14 @@ SockMessages::SockMessages(void): SettingsMessagesBase(client_address + 1) {
 				     "which may connect (separating different hostnames by spaces, but you do not "
 				     "need to specify localhost and your machine's hostname, as those are always "
 				     "permitted).  Such hostnames may be specified as an ordinary host name or in "
-				     "numeric dot notation.  Numeric dot notation may have * as a trailing wildcard, "
-				     "so for example 10.1.* will permit access for addresses 10.1.0.0 to 10.1.255.255.  "
+				     "numeric notation.  Numeric notation may have * as a trailing wildcard, so "
+				     "for example 10.1.* will permit access for addresses 10.1.0.0 to 10.1.255.255.  "
+				     "IPv6 numeric addresses may also have a trailing wildcard, but if so '::' may "
+				     "not be used in the address (but other forms of 0 suppression are acceptable).  "
 				     "You should run efax-gtk behind a firewall if you choose \"other\".");
   captions[client_address] = gettext("efax-gtk help: Socket connections allowed");
+  messages[ip_family] = gettext("This specifies the IP family to be used by the socket server.");
+  captions[ip_family] = gettext("efax-gtk help: IP family");
 }
 
 ReceiveMessages::ReceiveMessages(void): SettingsMessagesBase(exec + 1) {
