@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2010 Chris Vine
+/* Copyright (C) 2001 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -77,7 +77,7 @@ public:
 
 class SockMessages: private SettingsMessagesBase {
 public:
-  enum {run_server = 0, popup, port, client_address};
+  enum {run_server = 0, popup, port, client_address, ip_family};
   SockMessages(void);
 
   using SettingsMessagesBase::get_caption;
