@@ -366,7 +366,8 @@ std::pair<const char*, char* const*> SocketListDialog::get_view_file_parms(const
   }
   catch (Utf8::ConversionError&) {
     write_error("UTF-8 conversion error in SocketListDialog::get_view_file_parms()\n");
-    return std::pair<const char*, char* const*>(0,0);
+    return std::pair<const char*, char* const*>(static_cast<char*>(0),
+						static_cast<char* const*>(0));
   }
   
   if ((end_pos = view_cmd.find_first_of(' ')) != std::string::npos) { // we have parms
