@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 and 2009 Chris Vine
+/* Copyright (C) 2001 to 2005, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -26,17 +26,6 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/shared_ptr.h>
 
 
-namespace { // we put the functions in anonymous namespace in socket_list.cpp
-            // so they are not exported at link time
-namespace SocketListDialogCB {
-  extern "C" {
-    void socket_list_button_clicked(GtkWidget*, void*);
-    void socket_list_set_buttons(GtkTreeSelection*, void* data);
-    gboolean tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
-  }
-}
-}
-
 class SocketListDialog: public WinBase {
   static int is_socket_list;
   const int standard_size;
@@ -57,9 +46,8 @@ class SocketListDialog: public WinBase {
   void delete_parms(std::pair<const char*, char* const*>);
   void set_socket_list_item(const std::pair<std::string, unsigned int>&);
 public:
-  friend void SocketListDialogCB::socket_list_button_clicked(GtkWidget*, void*);
-  friend void SocketListDialogCB::socket_list_set_buttons(GtkTreeSelection*, void*);
-  friend gboolean SocketListDialogCB::tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
+  class CB;
+  friend class CB;
 
   Releaser releaser; // for MainWindow object
 
