@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 and 2006 and 2009 Chris Vine
+/* Copyright (C) 2004, 2006, 2009 and 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -17,9 +17,7 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
-namespace { // callback for internal use only
-
-void SocketNotifyDialogCB::socket_notify_button_clicked(GtkWidget* widget_p, void* data) {
+void SocketNotifyDialogCB::socket_notify_dialog_button_clicked(GtkWidget* widget_p, void* data) {
   SocketNotifyDialog* instance_p = static_cast<SocketNotifyDialog*>(data);
 
   if (widget_p == instance_p->number_button_p) {
@@ -33,26 +31,23 @@ void SocketNotifyDialogCB::socket_notify_button_clicked(GtkWidget* widget_p, voi
     }
   }
   else if (widget_p == instance_p->send_button_p) {
-    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+    gtk_widget_hide(GTK_WIDGET(instance_p->get_win()));
     instance_p->send_signals();
     instance_p->close();
   }
   else if (widget_p == instance_p->queue_button_p) {
-    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+    gtk_widget_hide(GTK_WIDGET(instance_p->get_win()));
     instance_p->queue_sig();
     instance_p->close();
   }
   else {
     write_error("Callback error in SocketNotifyDialogCB::socket_notify_button_clicked()\n");
-    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+    gtk_widget_hide(GTK_WIDGET(instance_p->get_win()));
     instance_p->queue_sig();
     instance_p->close();
   }
 }
 
-} // anonymous namespace
-
-
 SocketNotifyDialog::SocketNotifyDialog(const int size, 
 				       const std::pair<std::string, unsigned int>& fax_pair):
                                                WinBase(gettext("efax-gtk: print job received on socket"),
@@ -92,11 +87,11 @@ SocketNotifyDialog::SocketNotifyDialog(const int size,
   gtk_box_pack_start(GTK_BOX(window_box_p), button_box_p, false, false, standard_size/3);
 
   g_signal_connect(G_OBJECT(number_button_p), "clicked",
-		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_button_clicked), this);
+		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(send_button_p), "clicked",
-		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_button_clicked), this);
+		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_dialog_button_clicked), this);
   g_signal_connect(G_OBJECT(queue_button_p), "clicked",
-		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_button_clicked), this);
+		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_dialog_button_clicked), this);
 
 #if GTK_CHECK_VERSION(2,20,0)
   gtk_widget_set_can_default(number_button_p, true);
@@ -133,7 +128,7 @@ void SocketNotifyDialog::send_signals(void) {
 }
 
 void SocketNotifyDialog::on_delete_event(void) {
-  gtk_widget_hide_all(GTK_WIDGET(get_win()));
+  gtk_widget_hide(GTK_WIDGET(get_win()));
   queue_sig();
   close();
 }
