@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 to 2006 and 2009 Chris Vine
+/* Copyright (C) 2003 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -12,6 +12,7 @@ COPYING distributed with the source files.
 #include <algorithm>
 #include <ios>
 #include <ostream>
+#include <locale>
 #include <cstring>
 
 #include <sys/stat.h>
@@ -21,18 +22,16 @@ COPYING distributed with the source files.
 #include <arpa/inet.h>
 #include <unistd.h>
 #include <netdb.h>
-#include <netinet/in.h>
 #include <errno.h>
 #include <pthread.h>
 
+#include <glib.h>
+
 #include "socket_server.h"
 
 #include <c++-gtk-utils/shared_handle.h>
 #include <c++-gtk-utils/callback.h>
-
-#ifdef HAVE_STREAM_IMBUE
-#include <locale>
-#endif
+#include <c++-gtk-utils/mem_fun.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -72,11 +71,12 @@ SocketServer::SocketServer(void): server_running(false),
   read_queued_faxes();
 }
 
-void SocketServer::start(const std::string& port_, bool other_sock_client_address_) {
+void SocketServer::start(const std::string& port_, bool other_sock_client_address_, bool ipv6_) {
 
   if (!server_running) {
     port = port_;
     other_sock_client_address = other_sock_client_address_;
+    ipv6 = ipv6_;
     server_running = true;
 
     // now block off the signals for which we have set handlers so that the socket server
@@ -92,13 +92,10 @@ void SocketServer::start(const std::string& port_, bool other_sock_client_addres
     sigaddset(&sig_mask, SIGHUP);
     pthread_sigmask(SIG_BLOCK, &sig_mask, 0);
 
-    // temp_h is to keep gcc-2.95.3 and libstdc++2 happy as assigning the result
-    // of Thread::Thread::start() to socket_thread_h incorrectly reports an error
-    std::auto_ptr<Thread::Thread> temp_h =
+    thread =
       Thread::Thread::start(Callback::make(*this, &SocketServer::socket_thread),
 			    true);
-    socket_thread_h = temp_h;
-    if (!socket_thread_h.get()) {
+    if (!thread.get()) {
       write_error("Cannot start new socket thread, fax socket will not run\n");
       server_running = false;
     }
@@ -109,60 +106,105 @@ void SocketServer::start(const std::string& port_, bool other_sock_client_addres
 
 void SocketServer::socket_thread(void) {
 
+  bool running;
+
   // we don't want this thread to be cancelled until we reach accept_on_client()
   // make scope block for Thread::CancelBlock object
   {
     Thread::CancelBlock cancel_block;
+    if (ipv6) running = make_ipv6_socket();
+    else running = make_ipv4_socket();
 
-    if ((serve_sock_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-      write_error("Cannot create socket for socket server\n");
-      socket_error_notify();
-      return;
+    if (running) {
+      std::string message(gettext("Socket running on port "));
+      message += port;
+      message += '\n';
+      write_stdout_from_thread(message.c_str());
     }
-
-    sockaddr_in serve_address;
-    std::memset(&serve_address, 0, sizeof(serve_address));
-
-    serve_address.sin_family = AF_INET;
-    if (other_sock_client_address) serve_address.sin_addr.s_addr = htonl(INADDR_ANY);
-    else {
-      in_addr_t localhost_address = inet_addr("127.0.0.1");
-      if (localhost_address != (in_addr_t)(-1)) serve_address.sin_addr.s_addr = localhost_address;
-      else {
-	write_error("Cannot create network address for localhost\n");
-	socket_error_notify();
-	return;
-      }
-    }
-    serve_address.sin_port = htons(std::atoi(port.c_str()));
-    
-    if ((bind(serve_sock_fd, (sockaddr*)&serve_address, sizeof(serve_address))) < 0) {
-      write_error("Cannot bind to socket for socket server\n");
-      socket_error_notify();
-      return;
-    }
-
-    if ((listen(serve_sock_fd, 5)) < 0) {
-      write_error("Cannot listen on socket of socket server\n");
-      socket_error_notify();
-      return;
-    }
-
-    std::string message(gettext("Socket running on port "));
-    message += port;
-    message += '\n';
-    write_stdout_from_thread(message.c_str());
   } // end of CancelBlock block - thread can now be cancelled
-
-  // wait for clients to connect, and continue
+    // wait for clients to connect, and continue
   while (accept_on_client());
 }
 
+bool SocketServer::make_ipv4_socket(void) {
+
+  if ((serve_sock_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+    write_error("Cannot create IPv4 socket for socket server\n");
+    socket_error_notify();
+    return false;
+  }
+
+  sockaddr_in serve_address;
+  std::memset(&serve_address, 0, sizeof(serve_address));
+
+  serve_address.sin_family = AF_INET;
+  if (other_sock_client_address) serve_address.sin_addr.s_addr = htonl(INADDR_ANY);
+  else {
+    int res = inet_pton(AF_INET, "127.0.0.1", &(serve_address.sin_addr));
+    if (res <= 0) {
+      write_error("Cannot create IPv4 network address for localhost\n");
+      socket_error_notify();
+      return false;
+    }
+  }
+  serve_address.sin_port = htons(std::atoi(port.c_str()));
+    
+  if ((bind(serve_sock_fd, (sockaddr*)&serve_address, sizeof(serve_address))) < 0) {
+    write_error("Cannot bind to socket for socket server\n");
+    socket_error_notify();
+    return false;
+  }
+
+  if ((listen(serve_sock_fd, 5)) < 0) {
+    write_error("Cannot listen on socket of socket server\n");
+    socket_error_notify();
+    return false;
+  }
+  return true;
+}
+
+bool SocketServer::make_ipv6_socket(void) {
+
+  if ((serve_sock_fd = socket(AF_INET6, SOCK_STREAM, 0)) < 0) {
+    write_error("Cannot create IPv6 socket for socket server\n");
+    socket_error_notify();
+    return false;
+  }
+
+  sockaddr_in6 serve_address;
+  std::memset(&serve_address, 0, sizeof(serve_address));
+
+  serve_address.sin6_family = AF_INET6;
+  if (other_sock_client_address) serve_address.sin6_addr = in6addr_any;
+  else {
+    int res = inet_pton(AF_INET6, "::1", &(serve_address.sin6_addr));
+    if (res <= 0) {
+      write_error("Cannot create IPv6 network address for localhost\n");
+      socket_error_notify();
+      return false;
+    }
+  }
+  serve_address.sin6_port = htons(std::atoi(port.c_str()));
+    
+  if ((bind(serve_sock_fd, (sockaddr*)&serve_address, sizeof(serve_address))) < 0) {
+    write_error("Cannot bind to socket for socket server\n");
+    socket_error_notify();
+    return false;
+  }
+
+  if ((listen(serve_sock_fd, 5)) < 0) {
+    write_error("Cannot listen on socket of socket server\n");
+    socket_error_notify();
+    return false;
+  }
+  return true;
+}
+
 bool SocketServer::accept_on_client(void) {
 
   bool return_val = false;
 
-  sockaddr_in connect_address;
+  sockaddr_storage connect_address;
   std::memset(&connect_address, 0, sizeof(connect_address));
   socklen_t client_len = sizeof(connect_address);
 
@@ -177,7 +219,7 @@ bool SocketServer::accept_on_client(void) {
   // we only want thread cancellation to occur while accept() is blocking above
   Thread::CancelBlock cancel_block;
   if (connect_sock_fd >= 0
-      && is_valid_peer(connect_address)) {
+      && is_valid_peer(connect_address, client_len)) {
 
     std::string filename;
     {
@@ -277,7 +319,7 @@ void SocketServer::stop(void) {
   }
 
   if (server_running) {
-    socket_thread_h->cancel();
+    thread->cancel();
     cleanup();
   }
 }
@@ -293,7 +335,7 @@ void SocketServer::cleanup(void) {
     return;
   }
 
-  socket_thread_h->join();
+  thread->join();
 
   // this method is running in the main GUI thread so use stdout_message directly
   stdout_message(gettext("Closing the socket\n"));
@@ -305,8 +347,8 @@ void SocketServer::cleanup(void) {
   }
   server_running = false;
 
-  // the thread has ended so destroy the Thread object held by socket_thread_h
-  socket_thread_h.reset();
+  // the thread has ended so destroy the Thread object held by thread
+  thread.reset();
 }
 
 void SocketServer::add_file(const std::string& filename) {
@@ -476,59 +518,114 @@ void SocketServer::write_stdout_dispatcher_cb(void) {
   stdout_cond.signal();
 }
 
-bool SocketServer::is_valid_peer(const sockaddr_in& address) {
+bool SocketServer::is_valid_peer(const sockaddr_storage& address, socklen_t len) {
 
   bool return_val = false;
-  hostent* hostinfo_p;
   std::vector<std::string> peer_names;
 
-  // gethostbyaddr() is not guaranteed by IEEE Std 1003.1 to be thread safe -
-  // that doesn't matter as this program only uses it in this thread
-  if ((hostinfo_p = gethostbyaddr((const char*)&address.sin_addr,
-				  sizeof(address.sin_addr), AF_INET)) == 0) {
-    // inet_ntoa() is not guaranteed by IEEE Std 1003.1 to be thread safe -
-    // that doesn't matter as this program only uses it in this thread
-    const char* name_p = inet_ntoa(address.sin_addr);
-    if (name_p) peer_names.push_back(name_p);
-    else write_error("Cannot get hostname or numeric address of peer\n");
+  char buffer[INET6_ADDRSTRLEN + 512]; // general buffer which must be at least INET6_ADDRSTRLEN in
+                                       // size because we use it in inet_ntop() (INET6_ADDRSTRLEN
+                                       // is bigger than INET_ADDRSTRLEN)
+
+  // deal with the peer
+
+  // get unqualified domain name of peer (if any)
+  if (!getnameinfo((sockaddr*)&address, len,
+		   buffer, sizeof(buffer),
+		   0, 0, NI_NAMEREQD | NI_NOFQDN)) {
+    peer_names.push_back(buffer);
+  }
+  // get fully qualified domain name of peer
+  if (!getnameinfo((sockaddr*)&address, len,
+		   buffer, sizeof(buffer),
+		   0, 0, NI_NAMEREQD)) {
+    peer_names.push_back(buffer);
+  }
+  // get full list of peer's addresses in numeric string format
+  if (!getnameinfo((sockaddr*)&address, len,
+		   buffer, sizeof(buffer),
+		   0, 0, NI_NUMERICHOST)) {
+    // now obtain all the numeric addresses of the connecting peer
+    addrinfo hints;
+    std::memset(&hints, 0, sizeof(hints));
+    if (ipv6)
+      hints.ai_family = AF_INET6;
+    else
+      hints.ai_family = AF_INET;
+    hints.ai_socktype = SOCK_STREAM;
+
+    addrinfo* res;
+    if (getaddrinfo(buffer, 0, &hints, &res)) {
+      write_error("Cannot look up peer's numeric addresses\n");
+    }
+    else {
+      for (addrinfo* tmp = res; tmp; tmp = tmp->ai_next) {
+	void* addr;
+	if (ipv6) {
+	  sockaddr_in6* in = (sockaddr_in6*)tmp->ai_addr;
+	  addr = &(in->sin6_addr);
+        }
+	else {
+	  sockaddr_in* in = (sockaddr_in*)tmp->ai_addr;
+	  addr = &(in->sin_addr);
+        }
+	if (inet_ntop(tmp->ai_family, addr, buffer, sizeof(buffer))) {
+	  std::string tmp(normalise(buffer));
+	  if (!tmp.empty()) peer_names.push_back(tmp);
+	}
+      }
+      freeaddrinfo(res);
+    }   
+  }
+  else {
+    write_error("Cannot obtain numeric address of peer\n");
   }
 
-  else {
-    // store the values in hostinfo_p,
-    peer_names.push_back(hostinfo_p->h_name);
-    // load in any aliases
-    char** addrs_pp;
-    for (addrs_pp = hostinfo_p->h_aliases; *addrs_pp; addrs_pp++) {
-      peer_names.push_back(*addrs_pp);
-    }
-    // now load in the address(es) of the peer in numeric dot notation
-    // inet_ntoa() is not guaranteed by IEEE Std 1003.1 to be thread safe -
-    // that doesn't matter as this program only uses it in this thread
-    for (addrs_pp = hostinfo_p->h_addr_list; *addrs_pp; addrs_pp++) {
-      const char* name_p = inet_ntoa(*((in_addr*)*addrs_pp));
-      if (name_p) peer_names.push_back(name_p);
-    }
-  }
+  // now generate the valid names
 
   std::vector<std::string> valid_names;
   valid_names.push_back("localhost");           // localhost is always valid
-  valid_names.push_back("127.0.0.1");           // ditto
-
-  char my_hostname_p[256];
-  if (gethostname(my_hostname_p, 256) < 0) {
-    write_error("Cannot get our hostname\n");
+  if (ipv6) {
+    valid_names.push_back("0000:0000:0000:0000:0000:0000:0000:0001"); // we compare on expanded addresses
   }
   else {
-    my_hostname_p[255] = 0; // make sure it is null terminated
-    valid_names.push_back(my_hostname_p);       // our own hostname is also always valid
-    // gethostbyname is not guaranteed by IEEE Std 1003.1 to be thread safe -
-    // that doesn't matter as this program only uses it in this thread
-    hostent* my_info_p;
-    if ((my_info_p = gethostbyname(my_hostname_p)) == 0) {
-      write_error("Cannot get our fully qualified hostname\n");
-    }
-    else {
-      valid_names.push_back(my_info_p->h_name); // and our fq hostname is also valid
+    valid_names.push_back("127.0.0.1");         // ditto
+  }
+
+  if (gethostname(buffer, sizeof(buffer)) < 0) {
+    // we probably don't need to emit a warning with IPv6
+    if (!ipv6) write_error("Cannot get our hostname\n");
+  }
+  else {
+    buffer[sizeof(buffer) - 1] = 0;  // make sure it is null terminated
+    valid_names.push_back(buffer);   // our own hostname is also always valid
+    
+    addrinfo hints;
+    std::memset(&hints, 0, sizeof(hints));
+    if (ipv6) 
+      hints.ai_family = AF_INET6;
+    else
+      hints.ai_family = AF_INET;
+    hints.ai_socktype = SOCK_STREAM;
+
+    addrinfo* res;
+    if (!getaddrinfo(buffer, 0, &hints, &res)) {
+      for (addrinfo* tmp = res; tmp; tmp = tmp->ai_next) {
+	void* addr;
+	if (tmp->ai_family == AF_INET6) {
+	  sockaddr_in6* in = (sockaddr_in6*)tmp->ai_addr;
+	  addr = &(in->sin6_addr);
+        }
+	else {
+	  sockaddr_in* in = (sockaddr_in*)tmp->ai_addr;
+	  addr = &(in->sin_addr);
+        }
+	if (inet_ntop(tmp->ai_family, addr, buffer, sizeof(buffer))) {
+	  std::string tmp(normalise(buffer));
+	  if (!tmp.empty()) valid_names.push_back(tmp);
+	}
+      }
+      freeaddrinfo(res);
     }
   }
 
@@ -537,9 +634,14 @@ bool SocketServer::is_valid_peer(const sockaddr_in& address) {
   // lock the Prog_config object to stop it being modified or accessed in the initial
   // (GUI) thread while we are accessing it here
   Thread::Mutex::Lock lock(*prog_config.mutex_p);
-  std::copy(prog_config.permitted_clients_list.begin(),
-	    prog_config.permitted_clients_list.end(),
-	    std::back_inserter(valid_names));
+
+  // if there is a bad IPv6 address this will cause an empty string to be inserted
+  // but that's OK as addr_compare() checks for that
+  std::transform(prog_config.permitted_clients_list.begin(),
+		 prog_config.permitted_clients_list.end(),
+		 std::back_inserter(valid_names),
+		 Cgu::MemFun::make(*this, &SocketServer::normalise_checked));
+
   std::vector<std::string>::const_iterator peer_names_iter;
   std::vector<std::string>::const_iterator valid_names_iter;
   for (peer_names_iter = peer_names.begin();
@@ -560,7 +662,7 @@ bool SocketServer::addr_compare(const std::string& wild_str, const std::string&
   if (wild_str.empty()) return false;
 
   if (wild_str[wild_str.size() - 1] == '*'
-      && wild_str.find_first_not_of("0123456789.*") == std::string::npos) {
+      && wild_str.find_first_not_of("0123456789abcdefABCDEF.:*") == std::string::npos) {
 
     // take trailing '*' characters out of the comparison
     std::string::size_type length = wild_str.size();
@@ -572,3 +674,150 @@ bool SocketServer::addr_compare(const std::string& wild_str, const std::string&
   }
   return wild_str == in_str;
 }
+
+std::string SocketServer::ascii_to_lower(const std::string& in) {
+  std::locale loc("C");
+  Cgu::GcharScopedHandle str(g_strdup(in.c_str()));
+  std::use_facet<std::ctype<char> >(loc).tolower((char*)str.get(),
+						 (char*)str.get() + in.size());
+  return std::string((char*)str.get());
+}
+
+std::string SocketServer::normalise(const std::string& in) {
+
+  if (!ipv6) return in;
+
+  // if there is a wildcard, the '::' compression feature may not be used (but
+  // other forms of 0 suppression are OK)
+
+  // cursory first-sift check whether it appears to be a valid ipv6 address
+  // (this is mainly to ensure that the following implementation doesn't do
+  // anything too stupid, such as pass non-ASCII characters to ascii_to_lower(),
+  // than as a real check)
+  if (in.empty()
+      || in.size() > 45  // big enough for IPv4 mapped IPv6 addresses
+      || in.find_first_not_of("0123456789abcdefABCDEF:.*") != std::string::npos) {
+    write_error("Invalid IPv6 address entered\n");
+    return std::string();
+  }
+
+  std::size_t partition = in.find("::");
+  bool wildcard = in[in.size() - 1] == '*';
+  if (partition != std::string::npos && wildcard) {
+    write_error("Invalid wildcard supplied: cannot use wildcard with '::'\n");
+    return std::string();
+  }
+
+  // preconditions checked: now proceed
+
+  // deal with first part (this will never be entered if we have a
+  //  wildcard, begin with '::' or have no '::')
+  std::size_t start = 0; // first nibble of address word
+  std::size_t end;       // one past last nibble of address word
+
+  std::string part1;
+  if (partition                            // the address doesn't begin with '::'
+      && partition != std::string::npos) { // but we have a '::'
+    for (;;) {
+      bool finished = false;
+      end = in.find(':', start);
+      if (end == partition) {
+	finished = true;
+      }
+      for (int missing = 4 - static_cast<int>(end - start);
+	   missing > 0; --missing) {
+	part1 += '0';
+      }
+      for (; start < end; ++start) {
+	part1 += in[start];
+      }
+      if (!finished) {
+	part1 += ':';
+	start = end;
+	++start;
+      }
+      else {
+	start = end + 2;           // jump over '::' for next if block
+	break;
+      }
+    }
+  }
+
+  // deal with second part
+  int ipv4_count = 0; // IPv4-mapped IPv6 addresses consume the last two 16 bit
+                      // words for the IPv4 address and are of indeterminate length
+  std::string part2;
+  if (!partition) start = 2;       // address begins with '::' - jump over it
+  if (start != in.size()) {        // the address doesn't end with '::'
+    for (;;) {
+      bool finished = false;
+      end = in.find(':', start);
+      if (end == std::string::npos) {
+	finished = true;
+	end = in.size();           // one past last character
+	if (in.find('.', start) != std::string::npos)
+	  ipv4_count = end - start;    // number of characters in IPv4 address
+      }
+      if (!wildcard || !finished) {    // don't expand last element before wild card
+	for (int missing = 4 - static_cast<int>(end - start);
+	     missing > 0; --missing) { // The IPv4 address of IPv4-mapped IPv6 addresses will
+	                               // always occupy more than 4 characters so this will
+                                       // be passed over without a further test being necessary
+	  part2 += '0';
+	}
+      }
+      for (; start < end; ++start) {
+	part2 += in[start];
+      }
+      if (!finished) {
+	part2 += ':';
+	start = end;
+	++start;
+      }
+      else break;
+    }
+  }
+
+  // insert part1 in result
+  std::string result;
+  if (!part1.empty()) {
+    result += ascii_to_lower(part1);
+    result += ':'; // this will always be correct: if there is no '::' then part1 is empty.
+                   // If there is a '::', there is more to come
+  }
+
+  // now deal with the '::' expansion
+  if (!wildcard) {
+    int total;
+    // 40 not 39 - if the address ends with :: we only have part 1, which is an 
+    // exact multiple of 5 in size, and an empty part 2.  Discarding any remainder
+    // (which will be 1 if part 2 has something in it) is therefore correct.  (It
+    // could in fact be anything between 40 and 43.)
+    if (!ipv4_count) total = 40;
+    // for IPv4 mapping we substitute the IPv4 address for the last 2 words, and
+    // have 30 not 29 because the IPv6 element in part1 and part2 will always sum
+    // to a whole multiple of 5 in size with or without '::'
+    else total = 30;
+    for (int missing = (total - static_cast<int>(result.size() + part2.size() - ipv4_count))/5;
+	 missing > 0;
+       --missing) {
+      result += "0000";
+      if (missing > 1
+	  || !part2.empty())  // part2 cannot be empty for IPv4 mapped addresses
+	result += ':';
+    }
+  }
+
+  // add part2 in result
+  result += ascii_to_lower(part2);
+  return result;
+}
+
+// for IPv6 addresses which could be in either numeric or domain name
+// form (we might get these from prog_config.permitted_clients_list)
+std::string SocketServer::normalise_checked(const std::string& in) {
+  // check to see if 'in' is in domain name format - if so just return
+  // it (as it happens, this will also sift out ipv4 addresses)
+  if (std::count(in.begin(), in.end(), ':') < 2) return in;
+  return normalise(in);
+}
