@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 to 2005 and 2009 Chris Vine
+/* Copyright (C) 2003 to 2011 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -17,6 +17,7 @@ COPYING distributed with the source files.
 #include <memory>
 
 #include <netinet/in.h>
+#include <sys/socket.h>
 
 #include "utils/sem_sync.h"
 
@@ -25,6 +26,7 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/thread.h>
 #include <c++-gtk-utils/notifier.h>
 #include <c++-gtk-utils/emitter.h>
+#include <c++-gtk-utils/cgu_config.h>
 
 
 typedef std::list<std::pair<std::string, unsigned int> > FilenamesList;
@@ -34,6 +36,7 @@ class SocketServer {
 
   bool server_running;
   bool other_sock_client_address;
+  bool ipv6;
   int serve_sock_fd;
   unsigned int count;
   std::string working_dir;
@@ -49,7 +52,11 @@ class SocketServer {
   SharedPtr<FilenamesList> filenames_s;
   mutable Thread::Mutex filenames_mutex;
 
-  std::auto_ptr<Thread::Thread> socket_thread_h;
+#if !defined(CGU_USE_AUTO_PTR) && CGU_API_VERSION>=20
+  std::unique_ptr<Thread::Thread> thread;
+#else
+  std::auto_ptr<Thread::Thread> thread;
+#endif
 
   Thread::Mutex stdout_mutex;
   Thread::Mutex fax_to_send_mutex;
@@ -69,14 +76,19 @@ class SocketServer {
   std::pair<std::string, unsigned int> fax_to_send;
 
   void socket_thread(void);
+  bool make_ipv4_socket(void);
+  bool make_ipv6_socket(void);
   bool accept_on_client(void);
   void read_socket(int, int, const char*);
   void read_queued_faxes(void);
   void save_queued_faxes(void);
-  bool is_valid_peer(const sockaddr_in&);
-  bool addr_compare(const std::string&, const std::string&);
+  bool is_valid_peer(const sockaddr_storage&, socklen_t);
   void cleanup(void);
-
+  std::string normalise(const std::string&);
+  std::string normalise_checked(const std::string&);
+  static bool addr_compare(const std::string&, const std::string&);
+  static std::string ascii_to_lower(const std::string&);
+  
   // this is a concrete class not to be copied or assigned
   SocketServer(const SocketServer&);
   void operator=(const SocketServer&);
@@ -100,10 +112,11 @@ public:
   std::pair<SharedPtr<FilenamesList>, SharedPtr<Thread::Mutex::Lock> > get_filenames(void) const;
   std::pair<std::string, unsigned int> get_fax_to_send(void);  
   bool is_server_running(void) const {return server_running;}
+  bool is_ipv6(void) const {return ipv6;}
   std::string get_port(void) const {return port;}
   bool get_other_sock_client_address(void) const {return other_sock_client_address;}
   int get_count(void) const {return count;}
-  void start(const std::string& port, bool other_sock_client_address);
+  void start(const std::string& port, bool other_sock_client_address, bool ipv6);
   void stop(void);
   SocketServer(void);
   ~SocketServer(void) {if (server_running) stop();}
