@@ -18,68 +18,87 @@ COPYING distributed with the source files.
 #include <libintl.h>
 #endif
 
-namespace { // callbacks for internal use only
+class TrayIcon::CB {
+public:
+  static void tray_icon_activated(void* data) {
+    static_cast<TrayIcon*>(data)->activated();
+  }
 
-void TrayIconCB::tray_icon_activated(GtkStatusIcon*, void* data) {
-  static_cast<TrayIcon*>(data)->activated();
-}
+  static void tray_icon_popup(guint button,
+			      guint time,
+			      void* data) {
 
-void TrayIconCB::tray_icon_popup(GtkStatusIcon*, guint button,
-                                 guint time, void* data) {
-
-  TrayIcon* instance_p = static_cast<TrayIcon*>(data);
+    TrayIcon* instance_p = static_cast<TrayIcon*>(data);
   
-  int state;
-  instance_p->get_state(state);
-  if (state == EfaxController::inactive) {
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->stop_item_p), false);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_takeover_item_p), true);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_answer_item_p), true);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_standby_item_p), true);
+    int state;
+    instance_p->get_state(state);
+    if (state == EfaxController::inactive) {
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->stop_item_p), false);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_takeover_item_p), true);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_answer_item_p), true);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_standby_item_p), true);
+    }
+    else  {
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->stop_item_p), true);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_takeover_item_p), false);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_answer_item_p), false);
+      gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_standby_item_p), false);
+    }
+    gtk_menu_popup(GTK_MENU(instance_p->menu_h.get()), 0, 0, 0, 0,
+		   button, time);
   }
-  else  {
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->stop_item_p), true);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_takeover_item_p), false);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_answer_item_p), false);
-    gtk_widget_set_sensitive(GTK_WIDGET(instance_p->receive_standby_item_p), false);
-  }
-  gtk_menu_popup(GTK_MENU(instance_p->menu_h.get()), 0, 0, 0, 0,
-		 button, time);
-}
 
-void TrayIconCB::menu_item_activated(GtkMenuItem* item_p, void* data) {
-  TrayIcon* instance_p = static_cast<TrayIcon*>(data);
+  static void menu_item_activated(GtkMenuItem* item_p, void* data) {
+    TrayIcon* instance_p = static_cast<TrayIcon*>(data);
 
-  if (item_p == instance_p->list_received_faxes_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::list_received_faxes);
+    if (item_p == instance_p->list_received_faxes_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::list_received_faxes);
+    }
+    else if (item_p == instance_p->list_sent_faxes_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::list_sent_faxes);
+    }
+    else if (item_p == instance_p->redial_queue_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::redial_queue);
+    }
+    else if (item_p == instance_p->receive_takeover_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::receive_takeover);
+    }
+    else if (item_p == instance_p->receive_answer_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::receive_answer);
+    }
+    else if (item_p == instance_p->receive_standby_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::receive_standby);
+    }
+    else if (item_p == instance_p->stop_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::stop);
+    }
+    else if (item_p == instance_p->quit_item_p) {
+      instance_p->menu_item_chosen(TrayIcon::quit);
+    }
+    else {
+      write_error("Callback error in TrayIcon::CB::menu_item_activated()\n");
+    }
   }
-  else if (item_p == instance_p->list_sent_faxes_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::list_sent_faxes);
-  }
-  else if (item_p == instance_p->redial_queue_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::redial_queue);
-  }
-  else if (item_p == instance_p->receive_takeover_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::receive_takeover);
-  }
-  else if (item_p == instance_p->receive_answer_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::receive_answer);
-  }
-  else if (item_p == instance_p->receive_standby_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::receive_standby);
-  }
-  else if (item_p == instance_p->stop_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::stop);
-  }
-  else if (item_p == instance_p->quit_item_p) {
-    instance_p->menu_item_chosen(TrayIcon::quit);
-  }
-  else {
-    write_error("Callback error in TrayIconCB::menu_item_activated()\n");
-  }
-}
+};
 
-}  // anonymous namespace
+extern "C" {
+  static void tray_icon_activated_cb(GtkStatusIcon*,
+				     void* data) {
+    TrayIcon::CB::tray_icon_activated(data);
+  }
+
+  static void tray_icon_popup_cb(GtkStatusIcon*,
+				 guint button,
+				 guint time,
+				 void* data) {
+    TrayIcon::CB::tray_icon_popup(button, time, data);
+  }
+
+  static void menu_item_activated_cb(GtkMenuItem* item_p,
+				     void* data) {
+    TrayIcon::CB::menu_item_activated(item_p, data);
+  }
+} // extern "C"
 
 TrayIcon::TrayIcon(void): status_icon_h(gtk_status_icon_new()),
 			  menu_h(gtk_menu_new()) {
@@ -156,21 +175,21 @@ TrayIcon::TrayIcon(void): status_icon_h(gtk_status_icon_new()),
   gtk_widget_show(GTK_WIDGET(quit_item_p));
 
   g_signal_connect(G_OBJECT(list_received_faxes_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(list_sent_faxes_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(redial_queue_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(receive_takeover_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(receive_answer_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(receive_standby_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(stop_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
   g_signal_connect(G_OBJECT(quit_item_p), "activate",
-		   G_CALLBACK(TrayIconCB::menu_item_activated), this);
+		   G_CALLBACK(menu_item_activated_cb), this);
 
   { // scope block for the GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_scale_simple(prog_config.window_icon_h,
@@ -182,10 +201,10 @@ TrayIcon::TrayIcon(void): status_icon_h(gtk_status_icon_new()),
   set_tooltip_cb(gettext("Inactive"));
   
   g_signal_connect(G_OBJECT(status_icon_h.get()), "activate",
-                   G_CALLBACK(TrayIconCB::tray_icon_activated), this);
+                   G_CALLBACK(tray_icon_activated_cb), this);
 
   g_signal_connect(G_OBJECT(status_icon_h.get()), "popup-menu",
-                   G_CALLBACK(TrayIconCB::tray_icon_popup), this);
+                   G_CALLBACK(tray_icon_popup_cb), this);
 
 }
 
