@@ -17,17 +17,6 @@ COPYING distributed with the source files.
 #include <c++-gtk-utils/emitter.h>
 #include <c++-gtk-utils/gobj_handle.h>
 
-namespace { // we put the functions in anonymous namespace in tray_icon.cpp
-            // so they are not exported at link time
-namespace TrayIconCB {
-  extern "C" {
-    void menu_item_activated(GtkMenuItem*, void*);
-    void tray_icon_activated(GtkStatusIcon* , void*);
-    void tray_icon_popup(GtkStatusIcon*, guint, guint, void*);
-  }
-}
-}
-
 class TrayIcon {
 
   GobjHandle<GtkStatusIcon> status_icon_h;
@@ -49,15 +38,14 @@ public:
   enum MenuItem {quit, stop, receive_standby, receive_answer, receive_takeover,
 		 list_received_faxes, list_sent_faxes, redial_queue};
 
+  class CB;
+  friend class CB;
+
   Emitter activated;
   EmitterArg<int> menu_item_chosen;
   EmitterArg<int&> get_state;
   EmitterArg<int&> get_new_fax_count;
 
-  friend void TrayIconCB::menu_item_activated(GtkMenuItem*, void*);
-  friend void TrayIconCB::tray_icon_activated(GtkStatusIcon* , void*);
-  friend void TrayIconCB::tray_icon_popup(GtkStatusIcon*, guint, guint, void*);
-
   void set_tooltip_cb(const char* text);
   bool is_embedded(void);
 
