@@ -25,8 +25,11 @@ License as follows:
 #ifndef ICON_INFO_HANDLE_H
 #define ICON_INFO_HANDLE_H
 
-#include <gtk/gtkicontheme.h>
-#include "shared_handle.h"
+#include <gtk/gtk.h>
+
+#include <prog_defs.h>
+
+#include <c++-gtk-utils/shared_handle.h>
 
 class GtkIconInfoFree {
 public:
