@@ -26,7 +26,10 @@ License as follows:
 #define PANGO_LAYOUT_ITER_HANDLE_H
 
 #include <pango/pango-layout.h>
-#include "shared_handle.h"
+
+#include <prog_defs.h>
+
+#include <c++-gtk-utils/shared_handle.h>
 
 class PangoLayoutIterFree {
 public:
