@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 and 2005 Chris Vine
+/* Copyright (C) 2004 to 2010 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -27,15 +27,25 @@ License as follows:
 #include "selected_rows_handle.h"
 
 
-namespace { // callback for internal use only
-void SelectedRowsHandleCB::selected_rows_handle_fill_cb(GtkTreeModel*,
-							GtkTreePath* path_p,
-							GtkTreeIter*,
-							void* data) {
-  SelectedRowsHandle* instance_p = static_cast<SelectedRowsHandle*>(data);
-  instance_p->path_list.push_back(TreePathSharedHandle(gtk_tree_path_copy(path_p)));
-}
-}  // anonymous namespace
+class SelectedRowsHandle::CB {
+public:
+  static void fill(GtkTreePath* path_p,
+		   void* data) {
+    SelectedRowsHandle* instance_p = static_cast<SelectedRowsHandle*>(data);
+    instance_p->path_list.push_back(TreePathSharedHandle(gtk_tree_path_copy(path_p)));
+  }
+};
+
+// the GObject callback function with both C linkage
+// specification and internal linkage
+extern "C" {
+  static void srh_fill_cb(GtkTreeModel*,
+			  GtkTreePath* path_p,
+			  GtkTreeIter*,
+			  void* data) {
+    SelectedRowsHandle::CB::fill(path_p, data);
+  }
+} // extern "C"
 
 
 // this is a functor for converting a TreePathSharedHandle to a TreeRowRefSharedHandle
@@ -56,7 +66,7 @@ void SelectedRowsHandle::fill(const GtkTreeView* tree_view_p) {
   GtkTreeSelection* selection_p =
     gtk_tree_view_get_selection(const_cast<GtkTreeView*>(tree_view_p));
   gtk_tree_selection_selected_foreach(selection_p,
-				      SelectedRowsHandleCB::selected_rows_handle_fill_cb,
+				      srh_fill_cb,
 				      this);
 }
 
