@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 and 2005 Chris Vine
+/* Copyright (C) 2004 to 2010 Chris Vine
 
 The library comprised in this file or of which this file is part is
 distributed by Chris Vine under the GNU Lesser General Public
@@ -35,21 +35,14 @@ License as follows:
 typedef std::list<TreePathSharedHandle> RowPathList;
 typedef std::list<TreeRowRefSharedHandle> RowRefList;
 
-namespace { // we put the function in anonymous namespace in selected_rows_handle.cpp
-            // so it is not exported at link time
-namespace SelectedRowsHandleCB {
-  extern "C" void selected_rows_handle_fill_cb(GtkTreeModel*, GtkTreePath*, GtkTreeIter*, void*);
-}
-}
-
-
 class SelectedRowsHandle {
   RowPathList path_list;
 public:
-  friend void SelectedRowsHandleCB::selected_rows_handle_fill_cb(GtkTreeModel*,
-								 GtkTreePath*,
-								 GtkTreeIter*,
-								 void*);
+  // this helper class avoids exposing GObject callbacks with C
+  // linkage to the global namespace
+  class CB;
+  friend class CB;
+
   void fill(const GtkTreeView* tree_view_p);
   RowPathList::size_type size(void) const {return path_list.size();}
   bool is_empty(void) const {return path_list.empty();}
